import os, sys, logging
import warnings
import inspect
from webob.exc import HTTPNotFound

import tg
from tg import request_local
from tg.i18n import _get_translator
from tg.request_local import Request, Response

try: #pragma: no cover
    import pylons
    has_pylons = True
except:
    has_pylons = False


log = logging.getLogger(__name__)


class RequestLocals(object):
    __slots__ = ('response', 'request', 'app_globals',
                 'config', 'tmpl_context', 'translator',
                 'session', 'cache', 'url')


class TGApp(object):
    def __init__(self, config=None, **kwargs):
        """Initialize a base WSGI application

        Given an application configuration creates the WSGI
        application for it, if no configuration is provided
        then tg.config is used.

        TGApp constructor is also in charge of actually
        initializing application wrappers.
        """
        self.config = config = config or tg.config._current_obj()
        self.globals = config.get('tg.app_globals')
        self.package_name = config['package_name']
        self.lang = config.get('i18n.lang')

        if self.lang is None:
            backward_compatible_lang = config.get('lang')
            if backward_compatible_lang:
                warnings.warn('"lang" option has been renamed to "i18n.lang" and '
                              'will be removed in next major version.', DeprecationWarning)
                self.lang = backward_compatible_lang

        self.controller_classes = {}
        self.controller_instances = {}

        # Cache some options for use during requests
        self.strict_tmpl_context = self.config['tg.strict_tmpl_context']
        self.pylons_compatible = self.config.get('tg.pylons_compatible', True)

        self.resp_options = config.get('tg.response_options',
                                       dict(content_type='text/html',
                                            charset='utf-8',
                                            headers={'Cache-Control': 'no-cache',
                                                     'Pragma': 'no-cache',
                                                     'Content-Type': None,
                                                     'Content-Length': '0'}))

        self.wrapped_dispatch = self._dispatch
        for __, wrapper in self.config.get('application_wrappers', []):
            try:
                app_wrapper = wrapper(self.wrapped_dispatch, self.config)
                if getattr(app_wrapper, 'injected', True):
                    # if it conforms to the ApplicationWrapper ABC inject it only
                    # when an injected=True property is provided.
                    self.wrapped_dispatch = app_wrapper

                    # Force resolution of @cached_property, this speeds up requests
                    # and also acts as a prevention against race conditions on the
                    # property itself.
                    getattr(app_wrapper, 'next_handler', None)

            except TypeError:
                # backward compatibility with wrappers that didn't receive the config
                self.wrapped_dispatch = wrapper(self.wrapped_dispatch)

        if 'tg.root_controller' in self.config:
            self.controller_instances['root'] = self.config['tg.root_controller']

    def _setup_pylons_compatibility(self, environ, controller): #pragma: no cover
        """Updates environ to be backward compatible with Pylons"""
        try:
            environ['pylons.controller'] = controller
            environ['pylons.pylons'] = environ['tg.locals']

            self.config['pylons.app_globals'] = self.globals

            pylons.request = request_local.request
            pylons.cache = request_local.cache
            pylons.config = request_local.config
            pylons.app_globals = request_local.app_globals
            pylons.session = request_local.session
            pylons.translator = request_local.translator
            pylons.response = request_local.response
            pylons.tmpl_context = request_local.tmpl_context
        except ImportError:
            pass

    def __call__(self, environ, start_response):
        """Serve a WSGI Request"""
        # Hide outer middlewares when crash inside application itself
        __traceback_hide__ = 'before'

        testmode, context, registry = self._setup_app_env(environ)

        # Expose a path that simply registers the globals and preserves them
        # without doing much else
        if testmode is True and environ['PATH_INFO'] == '/_test_vars':
            registry.preserve(force=True)
            start_response('200 OK', [('Content-type', 'text/plain')])
            return ['DONE'.encode('utf-8')]

        controller = self._get_controller_instance('root')
        response = self.wrapped_dispatch(controller, environ, context)

        if testmode is True:
            environ['paste.testing_variables']['response'] = response

        try:
            if response is not None:
                return response(environ, start_response)

            raise Exception("No content returned by controller (Did you "
                            "remember to 'return' it?) in: %r" %
                            controller)
        finally:
            # Help Python collect ram a bit faster by removing the reference
            # cycle that the thread local objects cause
            del environ['tg.locals']
            if has_pylons and 'pylons.pylons' in environ: #pragma: no cover
                del environ['pylons.pylons']

    def _setup_app_env(self, environ):
        """Setup Request, Response and TurboGears context objects.

        Is also in charge of pushing TurboGears context into the
        paste registry and detect test mode. Returns whenever
        the testmode is enabled or not and the TurboGears context.
        """
        conf = self.config
        testing = False

        # Setup the basic global objects
        req = Request(environ)
        req._fast_setattr('_language', self.lang)
        req._fast_setattr('_response_type', None)

        resp_options = self.resp_options
        response = Response(
            content_type=resp_options['content_type'],
            charset=resp_options['charset'],
            headers=resp_options['headers']
        )

        # Setup the translator object
        translator = _get_translator(self.lang, tg_config=conf)

        if self.strict_tmpl_context:
            tmpl_context = TemplateContext()
        else:
            tmpl_context = AttribSafeTemplateContext()

        app_globals = self.globals

        locals = RequestLocals()
        locals.response = response
        locals.request = req
        locals.app_globals = app_globals
        locals.config = conf
        locals.tmpl_context = tmpl_context
        locals.translator = translator
        locals.session = environ.get('beaker.session')  # Usually None, unless middleware in place
        locals.cache = environ.get('beaker.cache')  # Usually None, unless middleware in place

        environ['tg.locals'] = locals

        # Register Global objects
        registry = environ['paste.registry']
        registry.register(request_local.config, conf)
        registry.register(request_local.context, locals)

        if 'paste.testing_variables' in environ:
            testing = True
            testenv = environ['paste.testing_variables']
            testenv['req'] = req
            testenv['response'] = response
            testenv['tmpl_context'] = tmpl_context
            testenv['app_globals'] = self.globals
            testenv['config'] = conf
            testenv['session'] = locals.session
            testenv['cache'] = locals.cache

        return testing, locals, registry

    @classmethod
    def class_name_from_module_name(cls, module_name):
        words = module_name.replace('-', '_').split('_')
        return ''.join(w.title() for w in words)

    @classmethod
    def lookup_controller(cls, config, controller):
        """Locates a controller by attempting to import it then grab
        the SomeController instance from the imported module.

        Override this to change how the controller object is found once
        the URL has been resolved.

        :param dict config: The configuration options for the application,
         usually this will be ``tg.config``.
        :param str controller: The controller name, this will be the name
         of the python module containing the controller.

        """
        root_module_path = config['paths']['root']
        base_controller_path = config['paths']['controllers']

        # remove the part of the path we expect to be the root part (plus one '/')
        assert base_controller_path.startswith(root_module_path)
        controller_path = base_controller_path[len(root_module_path)+1:]

        # attach the package
        full_module_name = '.'.join([config['package_name']] +
                                    controller_path.split(os.sep) +
                                    controller.split('/'))

        # Hide the traceback here if the import fails (bad syntax and such)
        __traceback_hide__ = 'before_and_this'

        __import__(full_module_name)
        module_name = controller.split('/')[-1]
        class_name = cls.class_name_from_module_name(module_name) + 'Controller'
        return getattr(sys.modules[full_module_name], class_name)

    def find_controller(self, controller):
        """Locates a controller for this TGApp.

        This is the same af :meth:`.lookup_controller` but will reuse
        configuration of the application and will cache results.

        :param str controller: The controller name, this will be the name
         of the python module containing the controller.
        """
        # Check to see if we've cached the class instance for this name
        if controller in self.controller_classes:
            return self.controller_classes[controller]

        mycontroller = self.lookup_controller(self.config, controller)

        self.controller_classes[controller] = mycontroller
        return mycontroller

    def _get_controller_instance(self, controller):
        # Check to see if we've cached the instance for this name
        try:
            return self.controller_instances[controller]
        except KeyError:
            mycontroller = self.find_controller(controller)

            # If it's a class, instantiate it
            if inspect.isclass(mycontroller):
                mycontroller = mycontroller()

            self.controller_instances[controller] = mycontroller
            return mycontroller

    def _dispatch(self, controller, environ, context):
        """Dispatches to a controller, the controller itself is expected
        to implement the routing system.
        """
        if not controller:
            return HTTPNotFound()

        # Setup pylons compatibility before calling controller
        if has_pylons and self.pylons_compatible:  # pragma: no cover
            self._setup_pylons_compatibility(environ, controller)

        # Controller is assumed to accept WSGI Environ and TG Context
        # and return a Response object.
        return controller(environ, context)


class TemplateContext(object):
    """Used by TurboGears as ``tg.tmpl_context``.

    It's just a plain python object with an improved representation
    to show all the attributes currently available into the object.

    """
    def __repr__(self):
        attrs = sorted((name, value)
                       for name, value in self.__dict__.items()
                       if not name.startswith('_'))
        parts = []
        for name, value in attrs:
            value_repr = repr(value)
            if len(value_repr) > 70:
                value_repr = value_repr[:60] + '...' + value_repr[-5:]
            parts.append(' %s=%s' % (name, value_repr))
        return '<%s.%s at %s%s>' % (
            self.__class__.__module__,
            self.__class__.__name__,
            hex(id(self)),
            ','.join(parts))

    def __getattr__(self, item):
        if item in ('form_values', 'form_errors'):
            warnings.warn('tmpl_context.form_values and tmpl_context.form_errors got deprecated '
                          'use request.validation instead', DeprecationWarning)
            return tg.request.validation[item[5:]]
        elif item == 'controller_url':
            warnings.warn('tmpl_context.controller_url got deprecated, '
                          'use request.controller_url instead', DeprecationWarning)
            return tg.request.controller_url

        raise AttributeError()


class AttribSafeTemplateContext(TemplateContext):
    """The ``tg.tmpl_context`` object, with lax attribute access (
    returns '' when the attribute does not exist)"""
    def __getattr__(self, name):
        try:
            return TemplateContext.__getattr__(self, name)
        except AttributeError:
            return ''



import logging
from ..i18n import sanitize_language_code, set_request_lang
from .._compat import string_type
from ..support.converters import asbool
from ..configuration.utils import coerce_config
from .base import ApplicationWrapper

log = logging.getLogger(__name__)


class I18NApplicationWrapper(ApplicationWrapper):
    """Provides Language detection from request and session.

    The session language(s) take priority over the request languages.

    Supported options which can be provided by config are:
        - ``i18n.enabled``: Whenever language detection is enabled or not.
        - ``i18n.lang``: Fallback language for the application, works both when language
          detection is enabled or disabled. If this is set and language detection is
          dislabled, the application will consider that all gettext wrapped strings must
          be translated to this language.
        - ``i18n.lang_session_key``: Session key from which to read the saved language
          (``tg_lang`` by default).
        - ``i18n.no_session_touch``: Avoid causing a session save when reading it to retrieve the
          favourite user language. This is ``False`` by default, setting it to ``False`` causes
          TurboGears to save and update the session for each request.

    """
    def __init__(self, handler, config):
        super(I18NApplicationWrapper, self).__init__(handler, config)

        options = {
            'enabled': False,
            'lang_session_key': 'tg_lang',
            'no_session_touch': False,
            'lang': None
        }
        options.update(coerce_config(config, 'i18n.',  {
            'enabled': asbool,
            'no_session_touch': asbool,
        }))

        self.enabled = options['enabled']
        self.options = options
        log.debug('i18n enabled: %s -> %s', self.enabled, self.options)

    @property
    def injected(self):
        return self.enabled

    def __call__(self, controller, environ, context):
        session_ = context.session
        if session_:
            session_existed = session_.accessed()
            # If session is available, we try to see if there are languages set
            languages = session_.get(self.options['lang_session_key'])
            if not session_existed and self.options['no_session_touch']:
                session_.__dict__['_sess'] = None

            if languages:
                if isinstance(languages, string_type):
                    languages = [languages]
            else:
                languages = []
        else:  # pragma: no cover
            languages = []

        languages.extend(map(sanitize_language_code, context.request.languages))
        set_request_lang(languages, tgl=context)

        return self.next_handler(controller, environ, context)



"""http exceptions for TurboGears

TurboGears http exceptions are inherited from WebOb http exceptions
"""
import webob
from webob.exc import *


class _HTTPMoveLazyLocation(object):
    """
    
    """
    def __init__(self, *args, **kw):
        try:
            kw['location'] = str(kw['location'])
        except KeyError:
            pass
        super(_HTTPMoveLazyLocation, self).__init__(*args, **kw)


class HTTPMovedPermanently(_HTTPMoveLazyLocation, webob.exc.HTTPMovedPermanently):
    """
    subclass of :class:`webob.exc.HTTPMovedPermanently` with
    support for lazy strings as location.

    This indicates that the requested resource has been assigned a new
    permanent URI and any future references to this resource SHOULD use
    one of the returned URIs.

    code: 301, title: Moved Permanently
    """


class HTTPFound(_HTTPMoveLazyLocation, webob.exc.HTTPFound):
    """
    subclass of :class:`webob.exc.HTTPFound` with
    support for lazy strings as location.

    This indicates that the requested resource resides temporarily under
    a different URI.

    code: 302, title: Found
    """


class HTTPTemporaryRedirect(_HTTPMoveLazyLocation, webob.exc.HTTPTemporaryRedirect):
    """
    subclass of :class:`webob.exc.HTTPTemporaryRedirect` with
    support for lazy strings as location.

    This indicates that if the client has performed a conditional GET
    request and access is allowed, but the document has not been
    modified, the server SHOULD respond with this status code.

    code: 304, title: Not Modified
    """



try:
    from hashlib import sha1
except ImportError:
    from sha1 import sha1

# avoid polluting module namespace
__all__ = ['codify']

def codify(value):
    string_hash = sha1(value.encode('ascii'))
    return string_hash.hexdigest()

def polluting_function(value):
    return "Template filter namespace has been POLLUTED"



# -*- coding: utf-8 -*-
"""
Utilities for lazy resolution of configurations.

Provides a bunch of tools to perform actions that need
the configuration to be in place when performed

"""

from logging import getLogger
log = getLogger(__name__)


class _ConfigMilestoneTracker(object):
    """Tracks actions that need to be performed
    when a specific configuration point is reached
    and required options are correctly initialized

    """
    def __init__(self, name):
        self.name = name
        self._actions = dict()
        self._reached = False
        self._keep_on_reset = []

    @property
    def reached(self):
        return self._reached

    def register(self, action, persist_on_reset=False):
        """Registers an action to be called on milestone completion.

        If milestone is already passed action is immediately called

        """
        if persist_on_reset:
            self._keep_on_reset.append(action)

        if self._reached:
            log.debug('%s milestone passed, calling %s directly', self.name, action)
            action()
        else:
            log.debug('Register %s to be called when %s reached', action, self.name)
            self._actions[id(action)] = action

    def reach(self):
        """Marks the milestone as reached.

        Runs the registered actions. Calling this
        method multiple times should lead to nothing.

        """
        log.debug('%s milestone reached', self.name)

        try:
            while True:
                try:
                    __, action = self._actions.popitem()
                    action()
                except KeyError:
                    break
        finally:
            self._reached = True

    def _reset(self):
        """This is just for testing purposes"""
        self._reached = False
        self._actions = dict()
        for action in self._keep_on_reset:
            self.register(action)


config_ready = _ConfigMilestoneTracker('config_ready')
renderers_ready = _ConfigMilestoneTracker('renderers_ready')
environment_loaded = _ConfigMilestoneTracker('environment_loaded')


def _reset_all():
    """Utility method for the test suite to reset milestones"""
    config_ready._reset()
    renderers_ready._reset()
    environment_loaded._reset()


def _reach_all():
    """Utility method for the test suite to reach all milestones"""
    config_ready.reach()
    renderers_ready.reach()
    environment_loaded.reach()



import atexit
from nose.tools import raises
from tests.test_configuration import PackageWithModel
from tg.configuration.utils import TGConfigError
from webtest import TestApp
from tg.wsgiapp import TGApp
from tg import expose, TGController
import tg
from tg.configuration import milestones, AppConfig
from tg.configuration.hooks import _TGGlobalHooksNamespace


class TestGlobalHooks:
    def setup(self):
        milestones._reset_all()

    def teardown(self):
        milestones._reset_all()
        tg.hooks = _TGGlobalHooksNamespace()  # Reset hooks

    def test_config_hooks(self):
        class RootController(TGController):
            @expose()
            def test(self):
                return 'HI!'

        visited_hooks = []
        def before_config_hook(app):
            visited_hooks.append('before_config')
            return app
        def after_config_hook(app):
            visited_hooks.append('after_config')
            return app
        def configure_new_app_hook(app):
            assert isinstance(app, TGApp)
            visited_hooks.append('configure_new_app')

        conf = AppConfig(minimal=True, root_controller=RootController())
        conf.register_hook('before_config', before_config_hook)
        conf.register_hook('after_config', after_config_hook)
        conf.register_hook('configure_new_app', configure_new_app_hook)
        app = conf.make_wsgi_app()
        app = TestApp(app)

        assert 'HI!' in app.get('/test')
        assert 'before_config' in visited_hooks
        assert 'after_config' in visited_hooks
        assert 'configure_new_app' in visited_hooks

    def test_controller_hooks_with_value(self):
        # Reset milestone so that registered hooks
        milestones._reset_all()

        class RootController(TGController):
            @expose()
            def test(self):
                return tg.hooks.notify_with_value('test_hook', 'BO',
                                                  controller=RootController.test)

        def value_hook(value):
            return value*2

        tg.hooks.register('test_hook', value_hook, controller=RootController.test)

        conf = AppConfig(minimal=True, root_controller=RootController())
        app = conf.make_wsgi_app()
        app = TestApp(app)

        resp = app.get('/test')
        assert 'BOBO' in resp, resp

    @raises(TGConfigError)
    def test_config_hooks_startup_on_controller(self):
        def f():
            pass

        tg.hooks.register('startup', None, controller=f)

    @raises(TGConfigError)
    def test_config_hooks_shutdown_on_controller(self):
        def f():
            pass

        tg.hooks.register('shutdown', None, controller=f)

    @raises(TGConfigError)
    def test_controller_wrapper_using_register(self):
        milestones.config_ready.reach()
        tg.hooks.register('controller_wrapper', None)

    @raises(TGConfigError)
    def test_global_controller_wrapper_after_milestone_reached(self):
        milestones.environment_loaded.reach()
        tg.hooks.wrap_controller(None)

    @raises(TGConfigError)
    def test_dedicated_controller_wrapper_after_milestone_reached(self):
        milestones.environment_loaded.reach()

        def f():
            pass

        tg.hooks.wrap_controller(None, controller=f)

    def test_startup_hook(self):
        # Temporary replace the hooks namespace so we register hooks only for this test
        original_hooks, tg.hooks = tg.hooks, _TGGlobalHooksNamespace()

        try:
            executed = []
            def func():
                executed.append(True)

            tg.hooks.register('startup', func)
            conf = AppConfig(minimal=True)
            app = conf.make_wsgi_app()
            assert True in executed, executed
        finally:
            tg.hooks = original_hooks

    def test_startup_hook_with_exception(self):
        # Temporary replace the hooks namespace so we register hooks only for this test
        original_hooks, tg.hooks = tg.hooks, _TGGlobalHooksNamespace()

        try:
            def func():
                raise Exception

            tg.hooks.register('startup', func)
            conf = AppConfig(minimal=True)
            app = conf.make_wsgi_app()
        finally:
            tg.hooks = original_hooks

    def test_shutdown_hook_callable(self):
        _registered_exit_funcs = []
        def _fake_atexit_register(what):
            _registered_exit_funcs.append(what)

        #Temporary replace atexit.register
        _real_register = atexit.register
        atexit.register = _fake_atexit_register

        # Temporary replace the hooks namespace so we register hooks only for this test
        original_hooks, tg.hooks = tg.hooks, _TGGlobalHooksNamespace()

        try:
            executed = []
            def func():
                executed.append(True)

            tg.hooks.register('shutdown', func)
            milestones.config_ready.reach()  # This forces hook registration
            assert func in tg.hooks._hooks['shutdown']
            assert len(_registered_exit_funcs), _registered_exit_funcs
            _registered_exit_funcs[0]()
            assert True in executed, executed
        finally:
            tg.hooks = original_hooks
            atexit.register = _real_register

    def test_disconnect_hooks(self):
        hook1_has_been_called = []
        def hook1_listener():
            hook1_has_been_called.append(True)

        class RootController(TGController):
            @expose()
            def test(self):
                tg.hooks.notify('custom_hook')
                return 'HI!'

        conf = AppConfig(minimal=True, root_controller=RootController())
        tg.hooks.register('custom_hook', hook1_listener)
        conf.package = PackageWithModel()
        app = conf.make_wsgi_app()
        app = TestApp(app)

        app.get('/test')
        app.get('/test')
        tg.hooks.disconnect('custom_hook', hook1_listener)
        app.get('/test')

        assert len(hook1_has_been_called) == 2, hook1_has_been_called

    def test_disconnect_hooks_multiple_listener(self):
        hook1_has_been_called = []
        def hook1_listener():
            hook1_has_been_called.append(True)

        hook2_has_been_called = []
        def hook2_listener():
            hook2_has_been_called.append(True)

        class RootController(TGController):
            @expose()
            def test(self):
                tg.hooks.notify('custom_hook', controller=RootController.test)
                return 'HI!'

        conf = AppConfig(minimal=True, root_controller=RootController())
        tg.hooks.register('custom_hook', hook1_listener)
        tg.hooks.register('custom_hook', hook2_listener)
        conf.package = PackageWithModel()
        app = conf.make_wsgi_app()
        app = TestApp(app)

        app.get('/test')
        app.get('/test')
        tg.hooks.disconnect('custom_hook', hook2_listener)
        app.get('/test')

        # Disconnecting an unregistered hook should do nothing.
        tg.hooks.disconnect('unregistered', hook1_listener)

        assert len(hook1_has_been_called) == 3, hook1_has_been_called
        assert len(hook2_has_been_called) == 2, hook2_has_been_called

    def test_disconnect_controller_hooks_multiple_listener(self):
        hook1_has_been_called = []
        def hook1_listener():
            hook1_has_been_called.append(True)

        hook2_has_been_called = []
        def hook2_listener():
            hook2_has_been_called.append(True)

        class RootController(TGController):
            @expose()
            def test(self):
                tg.hooks.notify('custom_hook', controller=RootController.test)
                return 'HI!'

            @expose()
            def test2(self):
                tg.hooks.notify('custom_hook', controller=RootController.test2)
                return 'HI'

        conf = AppConfig(minimal=True, root_controller=RootController())
        tg.hooks.register('custom_hook', hook1_listener, controller=RootController.test)
        tg.hooks.register('custom_hook', hook2_listener)
        conf.package = PackageWithModel()
        app = conf.make_wsgi_app()
        app = TestApp(app)

        app.get('/test')
        app.get('/test')
        app.get('/test2')

        assert len(hook1_has_been_called) == 2, hook1_has_been_called
        assert len(hook2_has_been_called) == 3, hook2_has_been_called

        tg.hooks.disconnect('custom_hook', hook1_listener, controller=RootController.test)

        app.get('/test')
        app.get('/test')

        assert len(hook1_has_been_called) == 2, hook1_has_been_called
        assert len(hook2_has_been_called) == 5, hook2_has_been_called



"""JSON encoding functions."""

import datetime
import decimal
import types

from json import JSONEncoder as _JSONEncoder
from tg.support.converters import asbool

from webob.multidict import MultiDict
from tg._compat import string_type
from tg.configuration.utils import GlobalConfigurable
from tg.util.sqlalchemy import dictify as dictify_sqla, is_saobject, is_query_result, is_query_row
from tg.util.ming import dictify as dictify_ming, is_mingobject, is_objectid

import logging
log = logging.getLogger(__name__)


class JsonEncodeError(Exception):
    """JSON Encode error"""


class JSONEncoder(_JSONEncoder, GlobalConfigurable):
    """TurboGears custom JSONEncoder.

    Provides support for encoding objects commonly used in TurboGears apps, like:

        - SQLAlchemy queries
        - Ming queries
        - Dates
        - Decimals
        - Generators

    Support for additional types is provided through the ``__json__`` method
    that will be called on the object by the JSONEncoder when provided and through
    the ability to register custom encoder for specific types using
    :meth:`.JSONEncoder.register_custom_encoder`.

    """
    CONFIG_NAMESPACE = 'json.'
    CONFIG_OPTIONS = {'isodates': asbool,
                      'allow_lists': asbool}

    def __init__(self, **kwargs):
        self._registered_types_map = {}
        self._registered_types_list = tuple()

        kwargs = self.configure(**kwargs)
        super(JSONEncoder, self).__init__(**kwargs)

    def configure(self, isodates=False, custom_encoders=None, allow_lists=False, **kwargs):
        """JSON encoder can be configured through :class:`.AppConfig` (``app_cfg.base_config``)
        using the following options:

        - ``json.isodates`` -> encode dates using ISO8601 format
        - ``json.custom_encoders`` -> List of tuples ``(type, encode_func)`` to register
          custom encoders for specific types.
        - ``json.allow_lists`` -> Allows lists to be encoded, this is usually disabled for
          security reasons due to JSON hijacking. See http://stackoverflow.com/questions/16289894
          for additional details.

        """
        self._isodates = isodates
        self._allow_lists = allow_lists
        if custom_encoders is not None:
            for type_, encoder in custom_encoders.items():
                self.register_custom_encoder(type_, encoder)
        return kwargs

    def register_custom_encoder(self, objtype, encoder):
        """Register a custom encoder for the given type.

        Instead of using standard behavior for encoding the given type to JSON, the
        ``encoder`` will used instead. ``encoder`` must be a callable that takes
        the object as argument and returns an object that can be encoded in JSON (usually a dict).

        """
        if objtype in self._registered_types_map:
            log.warning('%s type already registered for a custom encoder, replacing it', objtype)

        self._registered_types_map[objtype] = encoder
        # Append to head, so we find first the last registered types
        self._registered_types_list = (objtype, ) + self._registered_types_list

    def default(self, obj):
        if isinstance(obj, self._registered_types_list):
            # Minor optimization, enter loop only when we are instance of a supported type.
            for type_, encoder in self._registered_types_map.items():
                if isinstance(obj, type_):
                    return encoder(obj)
        elif hasattr(obj, '__json__') and callable(obj.__json__):
            return obj.__json__()
        elif isinstance(obj, (datetime.date, datetime.datetime, datetime.time)):
            if self._isodates:
                if isinstance(obj, (datetime.datetime, datetime.time)):
                    obj = obj.replace(microsecond=0)
                return obj.isoformat()
            else:
                return str(obj)
        elif isinstance(obj, decimal.Decimal):
            return float(obj)
        elif is_saobject(obj):
            return dictify_sqla(obj)
        elif is_mingobject(obj):
            return dictify_ming(obj)
        elif is_query_result(obj):
            return dict(rows=list(obj), count=obj.rowcount)
        elif is_query_row(obj):
            return dict(rows=dict(obj), count=1)
        elif is_objectid(obj):
            return str(obj)
        elif isinstance(obj, MultiDict):
            return obj.mixed()
        elif isinstance(obj, types.GeneratorType):
            return list(obj)
        else:
            return _JSONEncoder.default(self, obj)


_default_encoder = JSONEncoder.create_global()


def encode(obj, encoder=None, iterencode=False):
    """Return a JSON string representation of a Python object."""
    if encoder is None:
        encoder = _default_encoder

    encode_func = encoder.encode
    if iterencode:
        encode_func = encoder.iterencode

    if isinstance(obj, string_type):
        return encode_func(obj)

    if encoder._allow_lists is False:
        try:
            value = obj['test']
        except TypeError:
            if not hasattr(obj, '__json__') and not is_saobject(obj) and not is_mingobject(obj):
                raise JsonEncodeError('Your Encoded object must be dict-like.')
        except:
            pass

    return encode_func(obj)


def encode_iter(obj, encoder=None):
    """Encode object, yielding each string representation as available."""
    return encode(obj, encoder=encoder, iterencode=True)



# -*- coding: utf-8 -*-
from __future__ import absolute_import

try:
    import sqlalchemy
    from sqlalchemy.engine import ResultProxy, RowProxy
except ImportError:  # pragma: no cover
    sqlalchemy = None
    ResultProxy = None
    RowProxy = None


def is_saobject(obj):
    """Checks if the provided object is a SQLAlchemy model instance"""
    return sqlalchemy is not None and hasattr(obj, '_sa_class_manager')


def is_query_result(values):
    return ResultProxy is not None and isinstance(values, ResultProxy)


def is_query_row(obj):
    return RowProxy is not None and isinstance(obj, RowProxy)


def dictify(obj):
    """Converts a SQLAlchemy model instance to a dictionary"""
    if sqlalchemy is None:  # pragma: no cover
        raise RuntimeError('SQLAlchemy not available')

    if sqlalchemy.inspect(obj).detached:
        raise ValueError("SQLAlchemy instance '%r' must be attached to a session." % obj)

    props = {}
    for key in obj.__dict__:
        if not key.startswith('_sa_'):
            props[key] = getattr(obj, key)
    return props





"""
This is a striped down version of the Python Paste Registry Module
it is not meant to be used by itself, it's only purpose is to provide
global objects for TurboGears2.

# Original Module (c) 2005 Ben Bangert
# This module is part of the Python Paste Project and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php
"""

from tg.support.objectproxy import TurboGearsObjectProxy
from tg.support import NoDefault
import itertools, time
import threading as threadinglocal

__all__ = ['StackedObjectProxy', 'RegistryManager']


def _getboolattr(obj, attrname):
    try:
        return object.__getattribute__(obj, attrname)
    except AttributeError:  # pragma: no cover
        # This is here for compatibility with other middlewares that use
        # environ['paste.registry'] to register global objects.
        return None


class StackedObjectProxy(TurboGearsObjectProxy):
    """Track an object instance internally using a stack

    The StackedObjectProxy proxies access to an object internally using a
    stacked thread-local. This makes it safe for complex WSGI environments
    where access to the object may be desired in multiple places without
    having to pass the actual object around.

    New objects are added to the top of the stack with _push_object while
    objects can be removed with _pop_object.

    """
    def __init__(self, default=NoDefault, name="Default"):
        """Create a new StackedObjectProxy

        If a default is given, its used in every thread if no other object
        has been pushed on.

        """
        self.__dict__['____name__'] = name
        self.__dict__['____local__'] = threadinglocal.local()
        if default is not NoDefault:
            self.__dict__['____default_object__'] = default

    def _current_obj(self):
        """Returns the current active object being proxied to

        In the event that no object was pushed, the default object if
        provided will be used. Otherwise, a TypeError will be raised.

        """
        try:
            objects = self.____local__.objects
        except AttributeError:
            objects = None
        if objects:
            return objects[-1][0]
        else:
            obj = self.__dict__.get('____default_object__', NoDefault)
            if obj is not NoDefault:
                return obj
            else:
                raise TypeError(
                    'No object (name: %s) has been registered for this '
                    'thread' % self.____name__)

    def _push_object(self, obj):
        """Make ``obj`` the active object for this thread-local.

        This should be used like:

        .. code-block:: python

            obj = yourobject()
            module.glob = StackedObjectProxy()
            module.glob._push_object(obj)
            try:
                ... do stuff ...
            finally:
                module.glob._pop_object(conf)

        """
        try:
            self.____local__.objects.append((obj, False))
        except AttributeError:
            self.____local__.objects = []
            self.____local__.objects.append((obj, False))

    def _pop_object(self, obj=None):
        """Remove a thread-local object.

        If ``obj`` is given, it is checked against the popped object and an
        error is emitted if they don't match.

        """
        try:
            popped = self.____local__.objects.pop()
            popped_obj = popped[0]

            if obj and popped_obj is not obj:
                raise AssertionError(
                    'The object popped (%s) is not the same as the object '
                    'expected (%s)' % (popped_obj, obj))
        except (AttributeError, IndexError):
            raise AssertionError('No object has been registered for this thread')

    def _object_stack(self):
        """Returns all of the objects stacked in this container

        (Might return [] if there are none)
        """
        try:
            try:
                objs = self.____local__.objects
            except AttributeError:
                return []
            return objs[:]
        except AssertionError: #pragma: no cover
            return []

    def _preserve_object(self):
        try:
            object, preserved = self.____local__.objects[-1]
        except (AttributeError, IndexError):
            return

        self.____local__.objects[-1] = (object, True)

    @property
    def _is_preserved(self):
        try:
            objects = self.____local__.objects
        except AttributeError:
            return False

        if not objects:
            return False

        object, preserved = objects[-1]
        return preserved


class Registry(object):
    """Track objects and stacked object proxies for removal

    The Registry object is instantiated a single time for the request no
    matter how many times the RegistryManager is used in a WSGI stack. Each
    RegistryManager must call ``prepare`` before continuing the call to
    start a new context for object registering.

    Each context is tracked with a dict inside a list. The last list
    element is the currently executing context. Each context dict is keyed
    by the id of the StackedObjectProxy instance being proxied, the value
    is a tuple of the StackedObjectProxy instance and the object being
    tracked.

    """
    def __init__(self, enable_preservation=False):
        """Create a new Registry object

        ``prepare`` must still be called before this Registry object can be
        used to register objects.

        """
        self.reglist = []

        #preservation makes possible to keep around the objects
        #this is especially useful when debugging to avoid
        #discarding the objects after request completion.
        self.enable_preservation = enable_preservation

    def prepare(self):
        """Used to create a new registry context

        Anytime a new RegistryManager is called, ``prepare`` needs to be
        called on the existing Registry object. This sets up a new context
        for registering objects.

        """
        self.reglist.append({})

    def register(self, stacked, obj):
        """Register an object with a StackedObjectProxy"""

        if stacked is None:  # pragma: no cover
            # makes possible to disable registering for some
            # stacked objects by setting them to None.
            return

        myreglist = self.reglist[-1]
        stacked_id = id(stacked)
        if stacked_id in myreglist:
            stacked._pop_object(myreglist[stacked_id][1])
            del myreglist[stacked_id]

        # Avoid leaking memory on successive request when preserving objects
        if _getboolattr(stacked, '_is_preserved'):
            stacked._pop_object()

        stacked._push_object(obj)
        myreglist[stacked_id] = (stacked, obj)

    def cleanup(self):
        """Remove all objects from all StackedObjectProxy instances that
        were tracked at this Registry context"""
        for stacked, obj in self.reglist[-1].values():
            if not _getboolattr(stacked, '_is_preserved'):
                stacked._pop_object(obj)
        self.reglist.pop()

    def preserve(self, force=False):
        if not self.enable_preservation and force is False:
            return

        for stacked, obj in self.reglist[-1].values():
            if hasattr(stacked, '_preserve_object'):
                stacked._preserve_object()


class RegistryManager(object):
    """Creates and maintains a Registry context

    RegistryManager creates a new registry context for the registration of
    StackedObjectProxy instances. Multiple RegistryManager's can be in a
    WSGI stack and will manage the context so that the StackedObjectProxies
    always proxy to the proper object.

    The object being registered can be any object sub-class, list, or dict.

    Registering objects is done inside a WSGI application under the
    RegistryManager instance, using the ``environ['paste.registry']``
    object which is a Registry instance.

    """
    def __init__(self, application, streaming=False, preserve_exceptions=False):
        self.application = application
        self.streaming = streaming
        self.preserve_exceptions = preserve_exceptions

    def __call__(self, environ, start_response):
        app_iter = None
        reg = environ.setdefault('paste.registry', Registry(self.preserve_exceptions))
        reg.prepare()

        try:
            app_iter = self.application(environ, start_response)
        except:
            reg.preserve()
            reg.cleanup()
            raise
        else:
            # If we are streaming streaming_iter will cleanup things for us
            if not self.streaming:
                reg.cleanup()

        if self.streaming:
            return self.streaming_iter(reg, app_iter)

        return app_iter

    def streaming_iter(self, reg, data):
        try:
            for chunk in data:
                yield chunk
        except:
            reg.preserve()
            raise
        finally:
            if hasattr(data, 'close'):
                data.close()
            reg.cleanup()


class DispatchingConfig(StackedObjectProxy):
    """
    This is a configuration object that can be used globally,
    imported, have references held onto.  The configuration may differ
    by thread (or may not).

    Specific configurations are registered (and deregistered) either
    for the process or for threads.
    """
    # @@: What should happen when someone tries to add this
    # configuration to itself?  Probably the conf should become
    # resolved, and get rid of this delegation wrapper

    def __init__(self, name='DispatchingConfig'):
        super(DispatchingConfig, self).__init__(name=name)
        self.__dict__['_process_configs'] = []

    def push_thread_config(self, conf):
        """
        Make ``conf`` the active configuration for this thread.
        Thread-local configuration always overrides process-wide
        configuration.

        This should be used like::

            conf = make_conf()
            dispatching_config.push_thread_config(conf)
            try:
                ... do stuff ...
            finally:
                dispatching_config.pop_thread_config(conf)
        """
        self._push_object(conf)

    def pop_thread_config(self, conf=None):
        """
        Remove a thread-local configuration.  If ``conf`` is given,
        it is checked against the popped configuration and an error
        is emitted if they don't match.
        """
        self._pop_object(conf)

    def push_process_config(self, conf):
        """
        Like push_thread_config, but applies the configuration to
        the entire process.
        """
        self._process_configs.append(conf)

    def pop_process_config(self, conf=None):
        self._pop_from(self._process_configs, conf)

    def _pop_from(self, lst, conf):
        popped = lst.pop()
        if conf is not None and popped is not conf:
            raise AssertionError(
                "The config popped (%s) is not the same as the config "
                "expected (%s)"
                % (popped, conf))

    def _current_obj(self):
        try:
            return super(DispatchingConfig, self)._current_obj()
        except TypeError:
            if self._process_configs:
                return self._process_configs[-1]
            raise AttributeError(
                "No configuration has been registered for this process "
                "or thread")
    current = current_conf = _current_obj



"""Main Controller"""

from tg import expose, redirect, config
from tg.controllers import TGController

class RootController(TGController):
    @expose()
    def index(self):
        return "my foo"

    @expose()
    def config_test(self):
        return str(config)
    
    @expose()
    def config_attr_lookup(self):
        return str(config.render_functions)
    
    @expose()
    def config_dotted_values(self):
        return str(config.paths)
    
    @expose()
    def config_attr_set(self, foo):
        config.test_value = foo
        return str(config.test_value)
        
    @expose()
    def config_set_method(self):
        return str(config.get('pylons'))

    @expose()
    def config_dict_set(self, foo):
        config['test_value'] = foo
        return str(config.test_value)
        




import logging
import sys
from ..configuration.utils import coerce_config
from ..support.converters import asbool, aslist, asint
from .base import ApplicationWrapper

log = logging.getLogger(__name__)


class ErrorPageApplicationWrapper(ApplicationWrapper):
    """Given an Application it intercepts the response code and shows a custom page.

    Supported options are:

        - ``errorpage.enabled``: Whenever the custom error page is enabled or not.
        - ``errorpage.status_codes``: List of HTTP errors that should be trapped.
          By default 403, 404, 500.
        - ``errorpage.content_types``: List of Content-Types for which the custom error
          page should be displayed. IE: ``["text/html"]``. An empty list means all.
          An entry with value ``None`` means no content type provided.
          Default is ``["text/html", None]``.
        - ``errorpage.handle_exceptions``: Whenever exceptions should be trapped and
          treated as a 500 error or not. By default this is ``True`` when ``debug=false``.
        - ``errorapge.path``: Path of the controller should be displayed in case of
          errors. By default ``/error/document``.
    """

    def __init__(self, handler, config):
        super(ErrorPageApplicationWrapper, self).__init__(handler, config)

        options = {
            'enabled': False,
            'status_codes': tuple(),
            'handle_exceptions': not asbool(config.get('debug', False)),
            'path': '/error/document',
            'content_types': ["text/html", None]
        }
        options.update(coerce_config(config, 'errorpage.',  {
            'enabled': asbool,
            'status_codes': aslist,
            'handle_exceptions': asbool,
            'content_types': aslist
        }))

        self.handle_error_enabled = options['enabled']
        self.handle_status_codes = set(asint(s) for s in options['status_codes'])
        self.handle_exceptions = options['handle_exceptions']
        self.handle_error_path = options['path']
        self.handle_content_types = options['content_types']

        if self.handle_exceptions and 500 not in self.handle_status_codes:
            self.handle_status_codes.add(500)

        log.debug('ErrorPageApplicationWrapper enabled: %s -> %s',
                  self.handle_error_enabled, options)

    @property
    def injected(self):
        return self.handle_error_enabled

    def __call__(self, controller, environ, context):
        try:
            resp = self.next_handler(controller, environ, context)
        except:
            if self.handle_exceptions is False:
                raise
            # Provide crash details to backlash
            environ['backlash.exc_environ'] = environ.copy()
            environ['backlash.exc_info'] = sys.exc_info()
            # Force response to a 500 Error, otherwise it will be a 200
            resp = context.response
            resp.status_code = 500

        if not environ.get('tg.status_code_redirect', True):
            # status_code_redirect disabled per this request
            return resp

        status_code = resp.status_code
        content_type = resp.content_type
        log.debug('ErrorPageApplicationWrapper response: %s -> %s @ %s',
                  environ['PATH_INFO'], status_code, content_type)
        if status_code in self.handle_status_codes and \
                (not self.handle_content_types or content_type in self.handle_content_types):
            environ['tg.original_request'] = context.request.copy()
            environ['tg.original_response'] = resp

            environ['PATH_INFO'] = self.handle_error_path
            log.debug('ErrorPageApplicationWrapper serving %s:%s',
                      controller, self.handle_error_path)
            resp = self.next_handler(controller, environ, context)

        return resp



import copy
import logging, os
import gettext as _gettext
from gettext import NullTranslations, GNUTranslations
import warnings
import tg
from tg.util import lazify
from tg._compat import PY3, string_type

log = logging.getLogger(__name__)


class LanguageError(Exception):
    """Exception raised when a problem occurs with changing languages"""
    pass


def _parse_locale(identifier, sep='_'):
    """
    Took from Babel,
    Parse a locale identifier into a tuple of the form::

      ``(language, territory, script, variant)``

    >>> parse_locale('zh_CN')
    ('zh', 'CN', None, None)
    >>> parse_locale('zh_Hans_CN')
    ('zh', 'CN', 'Hans', None)

    The default component separator is "_", but a different separator can be
    specified using the `sep` parameter:

    :see: `IETF RFC 4646 <http://www.ietf.org/rfc/rfc4646.txt>`_
    """
    if '.' in identifier:
        # this is probably the charset/encoding, which we don't care about
        identifier = identifier.split('.', 1)[0]
    if '@' in identifier:
        # this is a locale modifier such as @euro, which we don't care about
        # either
        identifier = identifier.split('@', 1)[0]

    parts = identifier.split(sep)
    lang = parts.pop(0).lower()
    if not lang.isalpha():
        raise ValueError('expected only letters, got %r' % lang)

    script = territory = variant = None
    if parts:
        if len(parts[0]) == 4 and parts[0].isalpha():
            script = parts.pop(0).title()

    if parts:
        if len(parts[0]) == 2 and parts[0].isalpha():
            territory = parts.pop(0).upper()
        elif len(parts[0]) == 3 and parts[0].isdigit():
            territory = parts.pop(0)

    if parts:
        if len(parts[0]) == 4 and parts[0][0].isdigit() or\
           len(parts[0]) >= 5 and parts[0][0].isalpha():
            variant = parts.pop()

    if parts:
        raise ValueError('%r is not a valid locale identifier' % identifier)

    return lang, territory, script, variant


def gettext_noop(value):
    """Mark a string for translation without translating it. Returns
    value.
    """
    return value


def ugettext(value):
    """Mark a string for translation. Returns the localized unicode
    string of value.

    Mark a string to be localized as follows::

        _('This should be in lots of languages')

    """
    if PY3: #pragma: no cover
        return tg.translator.gettext(value)
    else:
        return tg.translator.ugettext(value)
lazy_ugettext = lazify(ugettext)


def ungettext(singular, plural, n):
    """Mark a string for translation. Returns the localized unicode
    string of the pluralized value.

    This does a plural-forms lookup of a message id. ``singular`` is
    used as the message id for purposes of lookup in the catalog, while
    ``n`` is used to determine which plural form to use. The returned
    message is a Unicode string.

    Mark a string to be localized as follows::

        ungettext('There is %(num)d file here', 'There are %(num)d files here',
                  n) % {'num': n}

    """
    if PY3: #pragma: no cover
        return tg.translator.ngettext(singular, plural, n)
    else:
        return tg.translator.ungettext(singular, plural, n)
lazy_ungettext = lazify(ungettext)


_TRANSLATORS_CACHE = {}
def _translator_from_mofiles(domain, mofiles, class_=None, fallback=False):
    """
    Adapted from python translation function in gettext module
    to work with a provided list of mo files
    """
    if class_ is None:
        class_ = GNUTranslations

    if not mofiles:
        if fallback:
            return NullTranslations()
        raise LanguageError('No translation file found for domain %s' % domain)

    result = None
    for mofile in mofiles:
        key = (class_, os.path.abspath(mofile))
        t = _TRANSLATORS_CACHE.get(key)
        if t is None:
            with open(mofile, 'rb') as fp:
                # Cache Translator to avoid reading it again
                t = _TRANSLATORS_CACHE.setdefault(key, class_(fp))

        t = copy.copy(t)
        if result is None:
            # Copy the translation object to be able to append fallbacks
            # without affecting the cached object.
            result = t
        else:
            result.add_fallback(t)

    return result


def _get_translator(lang, tgl=None, tg_config=None, **kwargs):
    """Utility method to get a valid translator object from a language name"""
    if tg_config:
        conf = tg_config
    else:
        if tgl:
            conf = tgl.config
        else:  # pragma: no cover
            #backward compatibility with explicit calls without
            #specifying local context or config.
            conf = tg.config.current_conf()

    if not lang:
        return NullTranslations()

    try:
        localedir = conf['localedir']
    except KeyError:  # pragma: no cover
        localedir = os.path.join(conf['paths']['root'], 'i18n')
    app_domain = conf['package'].__name__

    if not isinstance(lang, list):
        lang = [lang]

    mofiles = []
    supported_languages = []
    for l in lang:
        mo = _gettext.find(app_domain, localedir=localedir, languages=[l], all=False)
        if mo is not None:
            mofiles.append(mo)
            supported_languages.append(l)

    try:
        translator = _translator_from_mofiles(app_domain, mofiles, **kwargs)
    except IOError as ioe:
        raise LanguageError('IOError: %s' % ioe)

    translator.tg_lang = lang
    translator.tg_supported_lang = supported_languages

    return translator


def get_lang(all=True):
    """
    Return the current i18n languages used

    returns ``None`` if no supported language is available (no translations
    are in place) or a list of languages.

    In case ``all`` parameter is ``False`` only the languages for which
    the application is providing a translation are returned. Otherwise
    all the languages preferred by the user are returned.
    """
    if all is False:
        return getattr(tg.translator, 'tg_supported_lang', [])
    return getattr(tg.translator, 'tg_lang', [])


def add_fallback(lang, **kwargs):
    """Add a fallback language from which words not matched in other
    languages will be translated to.

    This fallback will be associated with the currently selected
    language -- that is, resetting the language via set_lang() resets
    the current fallbacks.

    This function can be called multiple times to add multiple
    fallbacks.
    """
    tgl = tg.request_local.context._current_obj()
    return tg.translator.add_fallback(_get_translator(lang, tgl=tgl, **kwargs))


sanitized_language_cache = {}
def sanitize_language_code(lang):
    """Sanitize the language code if the spelling is slightly wrong.

    For instance, 'pt-br' and 'pt_br' should be interpreted as 'pt_BR'.

    """
    try:
        lang = sanitized_language_cache[lang]
    except:
        orig_lang = lang

        try:
            lang = '_'.join(filter(None, _parse_locale(lang)[:2]))
        except ValueError:
            if '-' in lang:
                try:
                    lang = '_'.join(filter(None, _parse_locale(lang, sep='-')[:2]))
                except ValueError:
                    pass

        sanitized_language_cache[orig_lang] = lang

    return lang


def set_request_lang(languages, tgl=None):
    """Set the current request language(s) used for translations
    without touching the session language.

    languages should be a string or a list of strings.
    First lang will be used as main lang, others as fallbacks.

    """
    # the logging to the screen was removed because
    # the printing to the screen for every problem causes serious slow down.
    if not tgl:
        tgl = tg.request_local.context._current_obj()

    # Should only raise exceptions in case of IO errors,
    # so we let them propagate to the developer.
    tgl.translator = _get_translator(languages, tgl=tgl, fallback=True)

    # If the application has a set of supported translation
    # limit the formencode translations to those so that
    # we don't get the application in a language and
    # the errors in another one
    supported_languages = get_lang(all=False)
    if supported_languages:
        languages = supported_languages

    try:
        set_formencode_translation(languages, tgl=tgl)
    except LanguageError:
        pass


def set_temporary_lang(*args, **kwargs):
    warnings.warn("i18n.set_temporary_lang has been deprecated in favor of"
                  "i18n.set_request_lang and will be removed.", DeprecationWarning)
    return set_request_lang(*args, **kwargs)


def set_lang(languages, **kwargs):
    """Set the current language(s) used for translations
    in current call and session.

    languages should be a string or a list of strings.
    First lang will be used as main lang, others as fallbacks.

    """
    tgl = tg.request_local.context._current_obj()

    set_request_lang(languages, tgl)

    if tgl.session:
        tgl.session[tgl.config.get('lang_session_key', 'tg_lang')] = languages
        tgl.session.save()

FormEncodeMissing = '_MISSING_FORMENCODE'
formencode = None
_localdir = None

def set_formencode_translation(languages, tgl=None):
    """Set request specific translation of FormEncode."""
    global formencode, _localdir
    if formencode is FormEncodeMissing:  # pragma: no cover
        return

    if formencode is None:
        try:
            import formencode
            _localdir = formencode.api.get_localedir()
        except ImportError:  # pragma: no cover
            formencode = FormEncodeMissing
            return

    if not tgl:  # pragma: no cover
        tgl = tg.request_local.context._current_obj()

    try:
        formencode_translation = _gettext.translation('FormEncode',
                                                      languages=languages,
                                                      localedir=_localdir)
    except IOError as error:
        raise LanguageError('IOError: %s' % error)
    tgl.translator._formencode_translation = formencode_translation


# Idea stolen from Pylons
def _formencode_gettext(value):
    trans = ugettext(value)
    # Translation failed, try formencode
    if trans == value:
        try:
            fetrans = tg.translator._formencode_translation
        except (AttributeError, TypeError):
            # the translator was not set in the TG context
            # we are certainly in the test framework
            # let's make sure won't return something that is ok with the caller
            fetrans = None

        if not fetrans:
            fetrans = NullTranslations()

        translator_gettext = getattr(fetrans, 'ugettext', fetrans.gettext)
        trans = translator_gettext(value)

    return trans


__all__ = [
    "set_lang", "get_lang", "add_fallback",
    "set_request_lang", "set_temporary_lang",
    "ugettext", "lazy_ugettext", "ungettext", "lazy_ungettext"
]




from datetime import datetime
from email.utils import parsedate_tz, mktime_tz
import mimetypes
from time import gmtime, time
from os.path import normcase, normpath, join, isfile, getmtime, getsize
from webob.exc import HTTPNotFound, HTTPForbidden, HTTPBadRequest
from repoze.lru import LRUCache

_BLOCK_SIZE = 4096 * 64 # 256K

mimetypes.init()

class _FileIter(object):
    def __init__(self, file, block_size):
        self.file = file
        self.block_size = block_size

    def __iter__(self):
        return self

    def next(self):
        val = self.file.read(self.block_size)
        if not val:
            raise StopIteration
        return val

    __next__ = next # py3

    def close(self):
        self.file.close()

class FileServeApp(object):
    """
    Serves a static filelike object.
    """
    def __init__(self, path, cache_max_age):
        self.path = path

        try:
            self.last_modified = getmtime(path)
            self.content_length = getsize(path)
        except (IOError, OSError):
            self.path = None

        if self.path is not None:
            content_type, content_encoding = mimetypes.guess_type(path, strict=False)
            if content_type is None:
                content_type = 'application/octet-stream'

            self.content_type = content_type
            self.content_encoding = content_encoding

        if cache_max_age is not None:
            self.cache_expires = cache_max_age

    def generate_etag(self):
        return '"%s-%s"' % (self.last_modified, self.content_length)

    def parse_date(self, value):
        try:
            return mktime_tz(parsedate_tz(value))
        except (TypeError, OverflowError):
            raise HTTPBadRequest(("Received an ill-formed timestamp for %s: %s\r\n") % (self.path, value))

    @classmethod
    def make_date(cls, d):
        if isinstance(d, datetime):
            d = d.utctimetuple()
        else:
            d = gmtime(d)

        return '%s, %02d%s%s%s%s %02d:%02d:%02d GMT' % (
            ('Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun')[d.tm_wday],
            d.tm_mday, ' ',
            ('Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep',
             'Oct', 'Nov', 'Dec')[d.tm_mon - 1],
            ' ', str(d.tm_year), d.tm_hour, d.tm_min, d.tm_sec)


    def has_been_modified(self, environ, etag, last_modified):
        if environ['REQUEST_METHOD'] not in ('GET', 'HEAD'):
            return False

        unmodified = False

        modified_since = environ.get('HTTP_IF_MODIFIED_SINCE')
        if modified_since:
            modified_since = self.parse_date(modified_since)
            if last_modified and last_modified <= modified_since:
                unmodified = True

        if_none_match = environ.get('HTTP_IF_NONE_MATCH')
        if if_none_match and etag == if_none_match:
            unmodified = True

        return not unmodified

    def __call__(self, environ, start_response):
        try:
            file = open(self.path, 'rb')
        except (IOError, OSError, TypeError) as e:
            return HTTPForbidden('You are not permitted to view this file (%s)' % e)(environ, start_response)

        headers = []
        timeout = self.cache_expires
        etag = self.generate_etag()
        headers += [('Etag', '%s' % etag),
            ('Cache-Control', 'max-age=%d, public' % timeout)]

        if not self.has_been_modified(environ, etag, self.last_modified):
            file.close()
            start_response('304 Not Modified', headers)
            return []

        headers.extend((
            ('Expires', self.make_date(time() + timeout)),
            ('Content-Type', self.content_type),
            ('Content-Length', str(self.content_length)),
            ('Last-Modified', self.make_date(self.last_modified))
            ))
        start_response('200 OK', headers)
        return environ.get('wsgi.file_wrapper', _FileIter)(file, _BLOCK_SIZE)

INVALID_PATH_PARTS = set(['..', '.']).intersection

class StaticsMiddleware(object):
    def _adapt_path(self, path):
        return normcase(normpath(path))

    def __init__(self, app, root_dir, cache_max_age=3600):
        self.app = app
        self.cache_max_age = cache_max_age
        self.doc_root = self._adapt_path(root_dir)
        self.paths_cache = LRUCache(1024)

    def __call__(self, environ, start_response):
        full_path = environ['PATH_INFO']
        filepath = self.paths_cache.get(full_path)

        if filepath is None:
            path = full_path.split('/')
            if INVALID_PATH_PARTS(path):
                return HTTPNotFound('Out of bounds: %s' % environ['PATH_INFO'])(environ, start_response)
            filepath = self._adapt_path(join(self.doc_root, *path))
            self.paths_cache.put(full_path, filepath)

        if isfile(filepath):
            return FileServeApp(filepath, self.cache_max_age)(environ, start_response)

        return self.app(environ, start_response)




# -*- coding: utf-8 -*-
import tg
from tg.controllers import *
from tg.exceptions import HTTPFound
from nose.tools import eq_
from tests.base import setup_session_dir, teardown_session_dir
from tg.util import no_warn
from tg.util.webtest import test_context
from tg._compat import u_, string_type

def setup():
    setup_session_dir()
def teardown():
    teardown_session_dir()

def test_create_request():
    with test_context(None, '/', { 'SCRIPT_NAME' : '/xxx' }):
        eq_('http://localhost/xxx/hello', tg.request.relative_url('hello'))
        eq_('http://localhost/xxx', tg.request.application_url)

def test_approots():
    with test_context(None, '/subthing/',{ 'SCRIPT_NAME' : '/subthing' }):
        eq_("foo", url("foo"))
        eq_("/subthing/foo", url("/foo"))

def test_lowerapproots():
    with test_context(None, '/subthing/subsubthing/',{ 'SCRIPT_NAME' : '/subthing/subsubthing' }):
        eq_("/subthing/subsubthing/foo", url("/foo"))


@no_warn
def test_multi_values():
    with test_context(None, '/'):
        r = url("/foo", params=dict(bar=("asdf", "qwer")))
        assert r in \
                ["/foo?bar=qwer&bar=asdf", "/foo?bar=asdf&bar=qwer"], r
        r = url("/foo", params=dict(bar=[1,2]))
        assert r in \
                ["/foo?bar=1&bar=2", "/foo?bar=2&bar=1"], r


@no_warn
def test_unicode():
    """url() can handle unicode parameters"""
    with test_context(None, '/'):
        unicodestring =  u_('àèìòù')
        eq_(url('/', params=dict(x=unicodestring)),
            '/?x=%C3%A0%C3%A8%C3%AC%C3%B2%C3%B9'
            )


@no_warn
def test_list():
    """url() can handle list parameters, with unicode too"""
    with test_context(None, '/'):
        value = url('/', params=dict(foo=['bar', u_('à')])),
        assert '/?foo=bar&foo=%C3%A0' in value, value


@no_warn
def test_url_positional_params():
    with test_context(None, '/'):
        params = {'spamm': 'eggs'}
        result = url('/foo', params)
        assert 'spamm=eggs' in result

def test_url_with_params_key():
    with test_context(None, '/'):
        params = {'spamm': 'eggs'}
        result = url('/foo', params=params)
        assert 'spamm=eggs' in result

@no_warn
def test_url_strip_None():
    with test_context(None, '/'):
        params = {'spamm':'eggs', 'hamm':None }
        result = url('/foo', params=params)
        assert 'hamm' not in result, result

def test_lurl():
    with test_context(None, '/'):
        params = {'spamm':'eggs', 'hamm':None }
        assert url('/foo', params=params) == str(lurl('/foo', params=params))

@no_warn
def test_url_qualified():
    """url() can handle list parameters, with unicode too"""
    with test_context(None, '/'):
        value = url('/', qualified=True)
        assert value.startswith('http')

@no_warn
def test_lurl():
    """url() can handle list parameters, with unicode too"""
    with test_context(None, '/'):
        value = lurl('/lurl')
        assert not isinstance(value, string_type)
        assert value.startswith('/lurl')
        assert str(value) == repr(value) == value.id == value.encode('utf-8').decode('utf-8') == value.__html__()

def test_lurl_as_HTTPFound_location():
    with test_context(None, '/'):
        exc = HTTPFound(location=lurl('/lurl'))

        def _fake_start_response(*args, **kw):
            pass

        resp = exc({'PATH_INFO':'/',
                    'wsgi.url_scheme': 'HTTP',
                    'REQUEST_METHOD': 'GET',
                    'SERVER_NAME': 'localhost',
                    'SERVER_PORT': '80'}, _fake_start_response)
        assert b'resource was found at http://localhost:80/lurl' in resp[0]

def test_HTTPFound_without_location():
    exc = HTTPFound(add_slash=True)
 
    def _fake_start_response(*args, **kw):
        pass

    resp = exc({'PATH_INFO':'/here',
                'wsgi.url_scheme': 'HTTP',
                'REQUEST_METHOD': 'GET',
                'SERVER_NAME': 'localhost',
                'SERVER_PORT': '80'}, _fake_start_response)
    assert b'resource was found at http://localhost:80/here/' in resp[0]

@no_warn
def test_lurl_format():
    """url() can handle list parameters, with unicode too"""
    with test_context(None, '/'):
        value = lurl('/lurl/{0}')
        value = value.format('suburl')
        assert value == '/lurl/suburl', value

@no_warn
def test_lurl_add():
    """url() can handle list parameters, with unicode too"""
    with test_context(None, '/'):
        value = lurl('/lurl')
        value = value + '/suburl'
        assert value == '/lurl/suburl', value

@no_warn
def test_lurl_radd():
    """url() can handle list parameters, with unicode too"""
    with test_context(None, '/'):
        value = lurl('/lurl')
        value = '/suburl' + value
        assert value == '/suburl/lurl', value



from tg.configuration.utils import get_partial_dict


class Bunch(dict):
    """A dictionary that provides attribute-style access."""

    def __getitem__(self, key):
        return dict.__getitem__(self, key)

    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            return get_partial_dict(name, self, Bunch)

    __setattr__ = dict.__setitem__

    def __delattr__(self, name):
        try:
            del self[name]
        except KeyError:
            raise AttributeError(name)



from tests.test_stack import TestConfig, app_from_config
from tg.util import Bunch
from webtest import TestApp

from nose import SkipTest
from tg._compat import PY3


def setup_noDB(prefer_tw2=False):

    base_config = TestConfig(folder = 'rendering',
                     values = {'use_sqlalchemy': False,
                               # we want to test the new renderer functions
                               'use_legacy_renderer': False,
                               # in this test we want dotted names support
                               'use_dotted_templatenames': False,
                               'templating.genshi.method':'xhtml',
                               'prefer_toscawidgets2':prefer_tw2
                               }
                             )
    return app_from_config(base_config)


expected_fields = ['name="year"', 'name="title"']

def test_basic_form_rendering():
    if PY3: raise SkipTest()

    app = setup_noDB()
    resp = app.get('/form')
    assert "form" in resp

    for expected_field in expected_fields:
        assert expected_field in resp, resp

def test_tw2_form_rendering():
    app = setup_noDB(prefer_tw2=True)
    resp = app.get('/tw2form')
    assert "form" in resp

    for expected_field in expected_fields:
        assert expected_field in resp, resp



import hmac, base64, binascii
import warnings
from tg.support.objectproxy import TurboGearsObjectProxy
from tg.support.registry import StackedObjectProxy, DispatchingConfig
from tg.caching import cached_property

try:
    import cPickle as pickle
except ImportError: #pragma: no cover
    import pickle

try:
    from hashlib import sha1
except ImportError: #pragma: no cover
    import sha as sha1

from webob import Request as WebObRequest
from webob import Response as WebObResponse
from webob.request import PATH_SAFE
from webob.compat import url_quote as webob_url_quote, bytes_ as webob_bytes_


class Request(WebObRequest):
    """WebOb Request subclass

    The WebOb :class:`webob.Request` has no charset, or other defaults. This subclass
    adds defaults, along with several methods for backwards
    compatibility with paste.wsgiwrappers.WSGIRequest.

    """
    def _fast_setattr(self, name, value):
        object.__setattr__(self, name, value)

    def languages_best_match(self, fallback=None):
        al = self.accept_language
        try:
            items = [i for i, q in sorted(al._parsed, key=lambda iq: -iq[1])]
        except AttributeError:
            #NilAccept has no _parsed, here for test units
            items = []

        if fallback:
            for index, item in enumerate(items):
                if al._match(item, fallback):
                    items[index:] = [fallback]
                    break
            else:
                items.append(fallback)

        return items

    @cached_property
    def controller_state(self):
        warnings.warn("request.controller_state is now deprecated, please use"
                      "request.dispatch_state to access DispatchState for current request.",
                      DeprecationWarning, stacklevel=2)
        return self._controller_state

    @cached_property
    def dispatch_state(self):
        """Details and info about dispatcher that handled this request."""
        return self._controller_state

    @cached_property
    def controller_url(self):
        """Url of the current controller."""
        state = self._controller_state
        return '/'.join(state.path[:-len(state.remainder)])

    @cached_property
    def plain_languages(self):
        """Return the list of browser preferred languages"""
        return self.languages_best_match()

    @cached_property
    def languages(self):
        """Return the list of browser preferred languages ensuring that ``i18n.lang`` is listed."""
        return self.languages_best_match(self._language)

    @property
    def language(self):
        warnings.warn("request.language is now deprecated, use tg.config['i18n.lang'] to"
                      "read application fallback language.",
                      DeprecationWarning, stacklevel=2)
        return self._language

    @language.setter
    def language(self, value):
        warnings.warn("Setting request.language is now deprecated, use tg.i18n functions"
                      "to change the fallback language.",
                      DeprecationWarning, stacklevel=2)
        self._language = value

    @property
    def response_type(self):
        """Expected response content type when URL Extensions are enabled.

        In case URL Request Extension is enabled this will be the content type
        of the expected response. ``disable_request_extensions`` drives
        this is enabled or not.
        """
        return self._response_type

    @property
    def response_ext(self):
        """URL extension when URL Extensions are enabled.

        In case URL Request Extension is enabled this will be the extension of the url.
        ``disable_request_extensions`` drives this is enabled or not.
        """
        return self._response_ext

    def match_accept(self, mimetypes):
        return self.accept.best_match(mimetypes)

    def signed_cookie(self, name, secret):
        """Extract a signed cookie of ``name`` from the request

        The cookie is expected to have been created with
        ``Response.signed_cookie``, and the ``secret`` should be the
        same as the one used to sign it.

        Any failure in the signature of the data will result in None
        being returned.

        """
        cookie = self.cookies.get(name)
        if not cookie:
            return

        secret = secret.encode('ascii')
        try:
            sig, pickled = cookie[:40], base64.decodestring(cookie[40:].encode('ascii'))
        except binascii.Error: #pragma: no cover
            # Badly formed data can make base64 die
            return

        if hmac.new(secret, pickled, sha1).hexdigest() == sig:
            return pickle.loads(pickled)

    @cached_property
    def args_params(self):
        """Arguments used for dispatching the request.

        This mixes GET and POST arguments.
        """
        # This was: dict(((str(n), v) for n,v in self.params.mixed().items()))
        # so that keys were all strings making possible to use them as arguments.
        # Now it seems that all keys are always strings, did WebOb change behavior?
        return self.params.mixed()

    @property
    def quoted_path_info(self):
        """PATH used for dispatching the request."""
        bpath = webob_bytes_(self.path_info, self.url_encoding)
        return webob_url_quote(bpath, PATH_SAFE)

    def disable_error_pages(self):
        """Disable custom error pages for the current request.

        This will forward your response as is bypassing the :class:`.ErrorPageApplicationWrapper`
        """
        self.environ['tg.status_code_redirect'] = False

    def disable_auth_challenger(self):
        """Disable authentication challenger for current request.

        This will forward your response as is in case of 401 bypassing any
        repoze.who challenger.
        """
        self.environ['tg.skip_auth_challenge'] = True


class Response(WebObResponse):
    """WebOb Response subclass"""
    content = WebObResponse.body

    def wsgi_response(self):
        return self.status, self.headers, self.body

    def signed_cookie(self, name, data, secret, **kwargs):
        """Save a signed cookie with ``secret`` signature

        Saves a signed cookie of the pickled data. All other keyword
        arguments that ``WebOb.set_cookie`` accepts are usable and
        passed to the WebOb set_cookie method after creating the signed
        cookie value.

        """
        secret = secret.encode('ascii')

        pickled = pickle.dumps(data, pickle.HIGHEST_PROTOCOL)
        sig = hmac.new(secret, pickled, sha1).hexdigest().encode('ascii')
        cookie_value = sig + base64.encodestring(pickled)
        self.set_cookie(name, cookie_value, **kwargs)


config = DispatchingConfig()
context = StackedObjectProxy(name="context")


class TurboGearsContextMember(TurboGearsObjectProxy):
    """Member of the TurboGears request context.

    Provides access to turbogears context members
    like request, response, template context and so on

    """

    def __init__(self, name):
        self.__dict__['name'] = name

    def _current_obj(self):
        return getattr(context, self.name)


request = TurboGearsContextMember(name="request")
app_globals = TurboGearsContextMember(name="app_globals")
cache = TurboGearsContextMember(name="cache")
response = TurboGearsContextMember(name="response")
session = TurboGearsContextMember(name="session")
tmpl_context = TurboGearsContextMember(name="tmpl_context")
url = TurboGearsContextMember(name="url")
translator = TurboGearsContextMember(name="translator")

__all__ = ['app_globals', 'request', 'response', 'tmpl_context', 'session',
           'cache', 'translator', 'url', 'config']



"""Configuration Helpers for TurboGears 2"""
import os
import logging
import warnings
from copy import copy, deepcopy
import mimetypes
from collections import MutableMapping as DictMixin, deque
from tg._compat import import_module
from tg.appwrappers.identity import IdentityApplicationWrapper

from tg.support.middlewares import StaticsMiddleware, SeekableRequestBodyMiddleware, \
    DBSessionRemoverMiddleware
from tg.support.registry import RegistryManager
from tg.support.converters import asbool, asint, aslist
from tg.request_local import config as reqlocal_config

import tg
from tg.util import Bunch, DottedFileNameFinder
from tg.configuration import milestones
from tg.configuration.utils import TGConfigError, coerce_config, get_partial_dict, coerce_options, \
    DependenciesList
from tg.renderers.genshi import GenshiRenderer
from tg.renderers.json import JSONRenderer
from tg.renderers.jinja import JinjaRenderer
from tg.renderers.mako import MakoRenderer
from tg.renderers.kajiki import KajikiRenderer

from tg.appwrappers.i18n import I18NApplicationWrapper
from tg.appwrappers.caching import CacheApplicationWrapper
from tg.appwrappers.session import SessionApplicationWrapper
from tg.appwrappers.errorpage import ErrorPageApplicationWrapper
from tg.appwrappers.transaction_manager import TransactionApplicationWrapper
from tg.appwrappers.mingflush import MingApplicationWrapper

log = logging.getLogger(__name__)


class DispatchingConfigWrapper(DictMixin):
    """Wrapper for the Dispatching configuration.

    Simple wrapper for the DispatchingConfig object that provides attribute
    style access to the config dictionary.

    This class works by proxying all attribute and dictionary access to
    the underlying DispatchingConfig config object, which is an application local
    proxy that allows for multiple TG2 applications to live
    in the same process simultaneously, but to always get the right
    config data for the application that's requesting them.

    """

    def __init__(self, dict_to_wrap):
        """Initialize the object by passing in config to be wrapped"""
        self.__dict__['config_proxy'] = dict_to_wrap

    def __getitem__(self, key):
        return self.config_proxy.current_conf()[key]

    def __setitem__(self, key, value):
        self.config_proxy.current_conf()[key] = value

    def __getattr__(self, key):
        """Our custom attribute getter.

        Tries to get the attribute off the wrapped object first,
        if that does not work, tries dictionary lookup, and finally
        tries to grab all keys that start with the attribute and
        return sub-dictionaries that can be looked up.

        """
        try:
            return self.config_proxy.__getattribute__(key)
        except AttributeError:
            try:
                return self.config_proxy.current_conf()[key]
            except KeyError:
                return get_partial_dict(key, self.config_proxy.current_conf(), Bunch)

    def __setattr__(self, key, value):
        self.config_proxy.current_conf()[key] = value

    def __delattr__(self, name):
        try:
            del self.config_proxy.current_conf()[name]
        except KeyError:
            raise AttributeError(name)

    def __delitem__(self, key):
        self.__delattr__(key)

    def __len__(self):
        return len(self.config_proxy.current_conf())

    def __iter__(self):
        return iter(self.config_proxy.current_conf())

    def __repr__(self):
        return repr(self.config_proxy.current_conf())

    def keys(self):
        return self.config_proxy.keys()


defaults = {
    'debug': False,
    'package': None,
    'paths': {'root': None,
              'controllers': None,
              'templates': ['.'],
              'static_files': None},
    'tg.app_globals': None,
    'tg.strict_tmpl_context': True,
    'tg.pylons_compatible': True,
    'i18n.lang': None
}

# Push an empty config so all accesses to config at import time have something
# to look at and modify. This config will be merged with the app's when it's
# built in the paste.app_factory entry point.
reqlocal_config.push_process_config(deepcopy(defaults))

#Create a config object that has attribute style lookup built in.
config = DispatchingConfigWrapper(reqlocal_config)


def call_controller(tg_config, controller, remainder, params):
    return controller(*remainder, **params)


class _DeprecatedControllerWrapper(object):
    def __init__(self, controller_wrapper, config, next_wrapper):
        # Backward compatible old-way of configuring controller wrappers
        warnings.warn("Controller wrapper will now accept the configuration"
                      "as parameter when called instead of receiving it as"
                      "a constructor parameter, please refer to the documentation"
                      "to update your controller wrappers",
                      DeprecationWarning, stacklevel=3)

        def _adapted_next_wrapper(controller, remainder, params):
            return next_wrapper(tg.config._current_obj(),
                                controller, remainder, params)

        self.wrapper = controller_wrapper(config, _adapted_next_wrapper)

    def __call__(self, config, controller, remainder, params):
        return self.wrapper(controller, remainder, params)


class AppConfig(Bunch):
    """Class to store application configuration.

    This class should have configuration/setup information
    that is *necessary* for proper application function.
    Deployment specific configuration information should go in
    the config files (e.g. development.ini or deployment.ini).

    AppConfig instances have a number of methods that are meant to be
    overridden by users who wish to have finer grained control over
    the setup of the WSGI environment in which their application is run.

    This is the place to configure your application, database,
    transaction handling, error handling, etc.

    Configuration Options provided:

        - ``debug`` -> Enables / Disables debug mode. **Can be set from .ini file**
        - ``serve_static`` -> Enable / Disable serving static files. **Can be set from .ini file**
        - ``use_dotted_templatenames`` -> Use template names as packages in @expose instead of file paths.
          This is usually the default unless TG is started in Minimal Mode. **Can be set from .ini file**
        - ``registry_streaming`` -> Enable streaming of responses, this is enabled by default.
          **Can be set from .ini file**
        - ``paths`` -> Dictionary of directories where templates, static files and controllers are found::

            {
                'controllers': 'my/path/to/controlllers',
                'static_files': 'my/path/to/files',
                'templates': ['list/of/paths/to/templates']
            )
        - ``use_toscawidgets`` -> Enable ToscaWidgets1, this is deprecated.
        - ``use_toscawidgets2`` -> Enable ToscaWidgets2
        - ``prefer_toscawidgets2`` -> When both TW2 and TW1 are enabled prefer TW2. **Can be set from .ini file**
        - ``custom_tw2_config`` -> Dictionary of configuration options for TW2, refer to
          :class:`.tw2.core.middleware.Config` for available options.
        - ``auth_backend`` -> Authentication Backend, can be ``None``, ``sqlalchemy`` or ``ming``.
        - ``sa_auth`` -> Simple Authentication configuration dictionary.
          This is a Dictionary that contains the configuration options for ``repoze.who``,
          see :ref:`authentication` for available options. Basic options include:

            - ``cookie_secret`` -> Secret phrase used to verify auth cookies.
            - ``authmetadata`` -> Authentication and User Metadata Provider for TurboGears
            - ``post_login_url`` -> Redirect users here after login
            - ``post_logout_url`` -> Redirect users here when they logout
        - ``package`` -> Application Package, this is used to configure paths as being inside a python
        - ``app_globals`` -> Application Globals class, by default build from ``package.lib.app_globals``.
          package. Which enables serving templates, controllers, app globals and so on from the package itself.
        - ``helpers`` -> Template Helpers, by default ``package.lib.helpers`` is used.
        - ``model`` -> The models module (or object) where all the models, DBSession and init_models method are
           available. By default ``package.model`` is used.
        - ``renderers`` -> List of enabled renderers names.
        - ``default_renderer`` -> When not specified, use this renderer for templates.
        - ``auto_reload_templates`` -> Automatically reload templates when modified (disable this on production
          for a performance gain). **Can be set from .ini file**
        - ``use_ming`` -> Enable/Disable Ming as Models storage.
        - ``ming.url`` -> Url of the MongoDB database
        - ``ming.db`` -> If Database is not provided in ``ming.url`` it can be specified here.
        - ``ming.connection.*`` -> Options to configure the ming connection,
          refer to :func:`ming.datastore.create_datastore` for available options.
        - ``use_sqlalchemy`` -> Enable/Disable SQLalchemy as Models storage.
        - ``sqlalchemy.url`` -> Url of the SQLAlchemy database. Refer to :ref:`sqla_master_slave` for
          configuring master-slave urls.
    """
    CONFIG_OPTIONS = {
        'debug': asbool,
        'serve_static': asbool,
        'auto_reload_templates': asbool,
        'use_dotted_templatenames': asbool,
        'registry_streaming': asbool,
        'use_toscawidgets2': asbool,
        'prefer_toscawidgets2': asbool
    }

    def __init__(self, minimal=False, root_controller=None):
        """Creates some configuration defaults"""

        # Create a few bunches we know we'll use
        self.paths = Bunch()

        # Provide a default app_globals for single file applications
        self.app_globals = None
        self.helpers = None

        # And also very often...
        self.sa_auth = Bunch()

        # Set individual defaults
        self.auto_reload_templates = True
        self.auth_backend = None
        self.serve_static = not minimal

        self.renderers = []
        self.default_renderer = 'genshi'
        self.render_functions = Bunch()
        self.rendering_engines = {}
        self.rendering_engines_without_vars = set()
        self.rendering_engines_options = {}

        self.enable_routing_args = False
        self.disable_request_extensions = minimal

        self.use_ming = False
        self.use_sqlalchemy = False

        self['tm.enabled'] = not minimal

        self.use_toscawidgets = not minimal
        self.use_toscawidgets2 = False
        self.prefer_toscawidgets2 = False
        self.use_dotted_templatenames = not minimal
        self.registry_streaming = True

        self['session.enabled'] = not minimal
        self['cache.enabled'] = not minimal
        self['i18n.enabled'] = not minimal

        # Support Custom Error pages
        self.status_code_redirect = False  # Use old StatusCodeRedirect middleware
        self['errorpage.enabled'] = not minimal
        self['errorpage.status_codes'] = [403, 404]

        # Registry for functions to be called on startup/teardown
        self.call_on_startup = []
        self.call_on_shutdown = []
        self.controller_caller = call_controller
        self.controller_wrappers = []
        self.application_wrappers = DependenciesList()

        # override this variable to customize how the tw2 middleware is set up
        self.custom_tw2_config = {}

        # This is for minimal mode to set root controller manually
        if root_controller is not None:
            self['tg.root_controller'] = root_controller

        self.register_rendering_engine(JSONRenderer)
        self.register_rendering_engine(GenshiRenderer)
        self.register_rendering_engine(MakoRenderer)
        self.register_rendering_engine(JinjaRenderer)
        self.register_rendering_engine(KajikiRenderer)

        self.register_wrapper(I18NApplicationWrapper, after=True)
        self.register_wrapper(IdentityApplicationWrapper, after=True)
        self.register_wrapper(SessionApplicationWrapper, after=True)
        self.register_wrapper(CacheApplicationWrapper, after=True)
        self.register_wrapper(MingApplicationWrapper, after=True)
        self.register_wrapper(TransactionApplicationWrapper, after=True)
        self.register_wrapper(ErrorPageApplicationWrapper, after=True)

    def _get_root_module(self):
        root_module_path = self.paths['root']
        if not root_module_path:
            return None

        base_controller_path = self.paths['controllers']
        controller_path = base_controller_path[len(root_module_path)+1:]
        root_controller_module = '.'.join([self.package_name] + controller_path.split(os.sep) + ['root'])
        return root_controller_module

    def register_hook(self, hook_name, func):
        warnings.warn("AppConfig.register_hook is deprecated, "
                      "please use tg.hooks.register and "
                      "AppConfig.register_controller_wrapper instead", DeprecationWarning)

        if hook_name == 'controller_wrapper':
            tg.hooks.wrap_controller(func)
        else:
            tg.hooks.register(hook_name, func)

    def register_controller_wrapper(self, wrapper, controller=None):
        """Registers a TurboGears controller wrapper.

        Controller Wrappers are much like a **decorator** applied to
        every controller.
        They receive :class:`tg.configuration.AppConfig` instance
        as an argument and the next handler in chain and are expected
        to return a new handler that performs whatever it requires
        and then calls the next handler.

        A simple example for a controller wrapper is a simple logging wrapper::

            def controller_wrapper(app_config, caller):
                def call(*args, **kw):
                    try:
                        print 'Before handler!'
                        return caller(*args, **kw)
                    finally:
                        print 'After Handler!'
                return call

            base_config.register_controller_wrapper(controller_wrapper)

        It is also possible to register wrappers for a specific controller::

            base_config.register_controller_wrapper(controller_wrapper, controller=RootController.index)
        """
        if milestones.environment_loaded.reached:
            log.warning('Controller Wrapper %s registered after environment loaded'
                        'milestone has been reached, the wrapper will be used only'
                        'for future TGApp instances.', wrapper)

        log.debug("Registering %s controller wrapper for controller: %s",
                  wrapper, controller or 'ALL')

        if controller is None:
            self.controller_wrappers.append(wrapper)
        else:
            from tg.decorators import Decoration
            deco = Decoration.get_decoration(controller)
            deco._register_controller_wrapper(wrapper)

    def register_wrapper(self, wrapper, after=None):
        """Registers a TurboGears application wrapper.

        Application wrappers are like WSGI middlewares but
        are executed in the context of TurboGears and work
        with abstractions like Request and Respone objects.

        See :class:`tg.appwrappers.base.ApplicationWrapper` for
        complete definition of application wrappers.

        The ``after`` parameter defines their position into the
        wrappers chain. The default value ``None`` means they are
        executed in a middle point, so they run after the TurboGears
        wrappers like :class:`.ErrorPageApplicationWrapper` which
        can intercept their response and return an error page.

        Builtin TurboGears wrappers are usually registered with
        ``after=True`` which means they run furthest away from the
        application itself and can intercept the response of any
        other wrapper.

        Providing ``after=False`` means the wrapper will be registered
        near to the application itself (so wrappers registered at default
        position and with after=True will be able to see its response).

        ``after`` parameter can also accept an *application wrapper class*.
        In such case the registered wrapper will be registered right after
        the specified wrapper and so will be a little further from the
        application then the specified one (can see the response of the
        specified one).

        """
        if milestones.environment_loaded.reached:
            # Wrappers are consumed by TGApp constructor, and all the hooks available
            # after the milestone and that could register new wrappers are actually
            # called after TGApp constructors and so the wrappers wouldn't be applied.
            log.warning('Application Wrapper %s registered after environment loaded'
                        'milestone has been reached, the wrapper will be used only'
                        'for future TGApp instances.', wrapper)

        self.application_wrappers.add(wrapper, after=after)

    def register_rendering_engine(self, factory):
        """Registers a rendering engine ``factory``.

        Rendering engine factories are :class:`tg.renderers.base.RendererFactory`
        subclasses in charge of creating a rendering engine.

        """
        for engine, options in factory.engines.items():
            self.rendering_engines[engine] = factory
            self.rendering_engines_options[engine] = options
            if factory.with_tg_vars is False:
                self.rendering_engines_without_vars.add(engine)

    def _init_config(self, global_conf, app_conf):
        """Initialize the config object.

        Besides basic initialization, this method copies all the values
        in base_config into the ``tg.config`` objects.

        """
        # Load the mimetypes with its default types
        self.mimetypes = mimetypes.MimeTypes()

        try:
            self.package_name = self.package.__name__
        except AttributeError:
            self.package_name = None

        log.debug("Initializing configuration, package: '%s'", self.package_name)

        self._configure_package_paths()
        self._configure_renderers()
        self._configure_error_pages()
        self._configure_ming()
        self._configure_transaction_manager()
        self._configure_mimetypes()

        if self.prefer_toscawidgets2:
            self.use_toscawidgets = False
            self.use_toscawidgets2 = True

        conf = deepcopy(defaults)
        conf.update(self)
        conf.update(deepcopy(global_conf))
        conf.update(app_conf)
        conf.update(dict(app_conf=app_conf, global_conf=global_conf))

        # Coerce some options that are bool
        conf.update(coerce_options(
            conf,
            self.CONFIG_OPTIONS
        ))

        # Load the errorware configuration from the Paste configuration file
        # These all have defaults, and emails are only sent if configured and
        # if this application is running in production mode
        errorware = {}
        errorware['debug'] = conf['debug']
        if not errorware['debug']:
            errorware['debug'] = False

            trace_errors_config = coerce_config(conf, 'trace_errors.', {'enable': asbool,
                                                                        'smtp_use_tls': asbool,
                                                                        'dump_request_size': asint,
                                                                        'dump_request': asbool,
                                                                        'dump_local_frames': asbool,
                                                                        'dump_local_frames_count': asint})
            if not trace_errors_config:
                # backward compatibility
                warnings.warn("direct usage of error tracing options has been deprecated, "
                              "please specify them as trace_errors.option_name instad of directly "
                              "setting option_name. EXAMPLE: trace_errors.error_email", DeprecationWarning)

                trace_errors_config['error_email'] = conf.get('email_to')
                trace_errors_config['error_log'] = conf.get('error_log', None)
                trace_errors_config['smtp_server'] = conf.get('smtp_server', 'localhost')
                trace_errors_config['smtp_use_tls'] = asbool(conf.get('smtp_use_tls', False))
                trace_errors_config['smtp_username'] = conf.get('smtp_username')
                trace_errors_config['smtp_password'] = conf.get('smtp_password')
                trace_errors_config['error_subject_prefix'] = conf.get('error_subject_prefix', 'WebApp Error: ')
                trace_errors_config['from_address'] = conf.get('from_address', conf.get('error_email_from', 'turbogears@yourapp.com'))
                trace_errors_config['error_message'] = conf.get('error_message', 'An internal server error occurred')
            else:
                # Provide Defaults
                trace_errors_config.setdefault('error_subject_prefix',
                                               'WebApp Error: ')
                trace_errors_config.setdefault('error_message',
                                               'An internal server error occurred')

            errorware.update(trace_errors_config)

        conf['tg.errorware'] = errorware

        slowreqsware = coerce_config(conf, 'trace_slowreqs.', {'smtp_use_tls': asbool,
                                                               'dump_request_size': asint,
                                                               'dump_request': asbool,
                                                               'dump_local_frames': asbool,
                                                               'dump_local_frames_count': asint,
                                                               'enable': asbool,
                                                               'interval': asint,
                                                               'exclude': aslist})
        slowreqsware.setdefault('error_subject_prefix', 'Slow Request: ')
        slowreqsware.setdefault('error_message', 'A request is taking too much time')
        for erroropt in errorware:
            slowreqsware.setdefault(erroropt, errorware[erroropt])
        conf['tg.slowreqs'] = slowreqsware

        # Copy in some defaults
        if 'cache_dir' in conf:
            conf.setdefault('session.data_dir', os.path.join(conf['cache_dir'], 'sessions'))
            conf.setdefault('cache.data_dir', os.path.join(conf['cache_dir'], 'cache'))
        conf['tg.cache_dir'] = conf.pop('cache_dir', conf['app_conf'].get('cache_dir'))

        if not conf['paths']['static_files']:
            conf['serve_static'] = False

        conf['application_root_module'] = self._get_root_module()
        if conf['paths']['root']:
            conf['localedir'] = os.path.join(conf['paths']['root'], 'i18n')
        else:
            conf['i18n.enabled'] = False

        # Load conf dict into the global config object
        try:
            reqlocal_config.pop_process_config()
        except IndexError:  # pragma: no cover
            log.warn('No global config in place, at least defaults should have been here')
        finally:
            reqlocal_config.push_process_config(conf)

        self.after_init_config(conf)

        milestones.config_ready.reach()
        return conf

    def _configure_renderers(self):
        """Provides default configurations for renderers"""
        if not 'json' in self.renderers:
            self.renderers.append('json')

        if self.default_renderer not in self.renderers:
            first_renderer = self.renderers[0]
            log.warn('Default renderer not in renders, automatically switching to %s' % first_renderer)
            self.default_renderer = first_renderer

    def _configure_mimetypes(self):
        lookup = {'.json': 'application/json',
                  '.js': 'application/javascript'}
        lookup.update(config.get('mimetype_lookup', {}))

        for key, value in lookup.items():
            self.mimetypes.add_type(value, key)

    def _configure_ming(self):
        try:
            autoflush_enabled = self['ming.autoflush']
        except KeyError:
            autoflush_enabled = True

        self['ming.autoflush'] = self.use_ming and autoflush_enabled

    def _configure_error_pages(self):
        if (self.auth_backend is None and 401 not in self['errorpage.status_codes']):
            # If there's no auth backend configured which traps 401
            # responses we redirect those responses to a nicely
            # formatted error page
            self['errorpage.status_codes'] = list(self['errorpage.status_codes']) + [401]

        if self.status_code_redirect is True:
            # The codes TG should display an error page for. All other HTTP errors are
            # sent to the client or left for some middleware above us to handle
            self.handle_error_page = self['errorpage.enabled']
            self.handle_status_codes = self['errorpage.status_codes']
            self['errorpage.enabled'] = False

    def _configure_transaction_manager(self):
        if getattr(self, 'use_transaction_manager', False):
            warnings.warn("Transaction Manager middleware has been deprecated, "
                          "please use the tm.enabled=True option to switch "
                          "to the transaction application wrapper",
                          DeprecationWarning, stacklevel=2)
            self['tm.enabled'] = False

            if not self.use_sqlalchemy:
                log.debug('Disabling Transaction Manager as SQLAlchemy is not available')
                self.use_transaction_manager = False

    def _configure_package_paths(self):
        try:
            self.package
        except AttributeError:
            # if we don't have a specified package, don't try
            # helpers from the package expect the user to specify them.
            paths = Bunch()
        else:
            root = os.path.dirname(os.path.abspath(self.package.__file__))
            paths = Bunch(
                root=root,
                controllers=os.path.join(root, 'controllers'),
                static_files=os.path.join(root, 'public'),
                templates=[os.path.join(root, 'templates')]
            )

        # If the user defined custom paths, then use them instead of the
        # default ones:
        paths.update(self.paths)

        # Ensure all paths are set, load default ones otherwise
        for key, val in defaults['paths'].items():
            paths.setdefault(key, val)

        self.paths = paths

    def after_init_config(self, conf):
        """
        Override this method to set up configuration variables at the application
        level.  This method will be called after your configuration object has
        been initialized on startup.  Here is how you would use it to override
        the default setting of tg.strict_tmpl_context ::

            from tg.configuration import AppConfig

            class MyAppConfig(AppConfig):
                def after_init_config(self, conf):
                    conf['tg.strict_tmpl_context'] = False

            base_config = MyAppConfig()

        """

    def _setup_helpers_and_globals(self, conf):
        """Add helpers and globals objects to the ``conf``.

        Override this method to customize the way that ``app_globals`` and ``helpers``
        are setup. TurboGears expects them to be available in ``conf`` dictionary
        as ``tg.app_globals`` and ``helpers``.
        """
        # Setup AppGlobals
        gclass = conf.pop('app_globals', None)
        if gclass is None:
            try:
                gclass = conf['package'].lib.app_globals.Globals
            except AttributeError:
                pass

        if gclass is None:
            try:
                app_globals_mod = import_module('.lib.app_globals', package=self.package.__name__)
                gclass = getattr(app_globals_mod, 'Globals')
            except (ImportError, AttributeError):
                pass

        if gclass is None:
            log.warn('app_globals not provided and lib.app_globals.Globals is not available.')
            gclass = Bunch

        g = gclass()

        g.dotted_filename_finder = DottedFileNameFinder()
        conf['tg.app_globals'] = g

        if conf.get('tg.pylons_compatible', True):
            conf['pylons.app_globals'] = g

        # Setup Helpers
        h = conf.get('helpers', None)
        if h is None:
            try:
                h = conf['package'].lib.helpers
            except AttributeError:
                pass

        if h is None:
            try:
                h = import_module('.lib.helpers', package=self.package.__name__)
            except (ImportError, AttributeError):
                pass

        if h is None:
            log.warn('helpers not provided and lib.helpers is not available.')
            h = Bunch()

        conf['helpers'] = h

    def _setup_persistence(self, conf):
        """Configures application persistence model"""
        if conf['use_sqlalchemy']:
            self._setup_sqlalchemy(conf)
        if conf['use_ming']:
            self._setup_ming(conf)

    def _setup_ming(self, conf):
        """Setup MongoDB database engine using Ming"""
        try:
            from ming import create_datastore
            def create_ming_datastore(url, database, **kw):
                if database and url[-1] != '/':
                    url += '/'
                ming_url = url + database
                return create_datastore(ming_url, **kw)
        except ImportError: #pragma: no cover
            from ming.datastore import DataStore
            def create_ming_datastore(url, database, **kw):
                return DataStore(url, database=database, **kw)

        def mongo_read_pref(value):
            from pymongo.read_preferences import ReadPreference
            return getattr(ReadPreference, value)

        datastore_options = coerce_config(conf, 'ming.connection.', {'max_pool_size':asint,
                                                                     'network_timeout':asint,
                                                                     'tz_aware':asbool,
                                                                     'safe':asbool,
                                                                     'journal':asbool,
                                                                     'wtimeout':asint,
                                                                     'fsync':asbool,
                                                                     'ssl':asbool,
                                                                     'read_preference':mongo_read_pref})
        datastore_options.pop('host', None)
        datastore_options.pop('port', None)

        datastore = create_ming_datastore(conf['ming.url'],
                                          conf.get('ming.db', ''),
                                          **datastore_options)
        conf['tg.app_globals'].ming_datastore = datastore

        try:
            package_models = conf['package'].model
        except AttributeError:
            package_models = None

        model = conf.get('model', package_models)
        if model is None:
            raise TGConfigError('Ming enabled, but no models provided')

        ming_session = model.init_model(datastore)
        if ming_session is not None:
            # If init_model returns a specific session, keep it around
            # as the MongoDB Session.
            conf['MingSession'] = ming_session

        if 'DBSession' not in conf:
            # If the user hasn't specified a default session, assume
            # he/she uses the default DBSession in model
            conf['DBSession'] = model.DBSession

    def _setup_sqlalchemy(self, conf):
        """Setup SQLAlchemy database engine.

        The most common reason for modifying this method is to add
        multiple database support.  To do this you might modify your
        app_cfg.py file in the following manner::

            from tg.configuration import AppConfig, config
            from myapp.model import init_model

            # add this before base_config =
            class MultiDBAppConfig(AppConfig):
                def setup_sqlalchemy(self):
                    '''Setup SQLAlchemy database engine(s)'''
                    from sqlalchemy import engine_from_config
                    engine1 = engine_from_config(config, 'sqlalchemy.first.')
                    engine2 = engine_from_config(config, 'sqlalchemy.second.')
                    # engine1 should be assigned to sa_engine as well as your first engine's name
                    config['tg.app_globals'].sa_engine = engine1
                    config['tg.app_globals'].sa_engine_first = engine1
                    config['tg.app_globals'].sa_engine_second = engine2
                    # Pass the engines to init_model, to be able to introspect tables
                    init_model(engine1, engine2)

            #base_config = AppConfig()
            base_config = MultiDBAppConfig()

        This will pull the config settings from your .ini files to create the necessary
        engines for use within your application.  Make sure you have a look at :ref:`multidatabase`
        for more information.

        """
        from sqlalchemy import engine_from_config

        balanced_master = conf.get('sqlalchemy.master.url')
        if not balanced_master:
            engine = engine_from_config(conf, 'sqlalchemy.')
        else:
            engine = engine_from_config(conf, 'sqlalchemy.master.')
            conf['balanced_engines'] = {'master':engine,
                                        'slaves':{},
                                        'all':{'master':engine}}

            all_engines = conf['balanced_engines']['all']
            slaves = conf['balanced_engines']['slaves']
            for entry in conf.keys():
                if entry.startswith('sqlalchemy.slaves.'):
                    slave_path = entry.split('.')
                    slave_name = slave_path[2]
                    if slave_name == 'master':
                        raise TGConfigError('A slave node cannot be named master')
                    slave_config = '.'.join(slave_path[:3])
                    all_engines[slave_name] = slaves[slave_name] = engine_from_config(conf, slave_config+'.')

            if not conf['balanced_engines']['slaves']:
                raise TGConfigError('When running in balanced mode your must specify at least a slave node')

        # Pass the engine to initmodel, to be able to introspect tables
        conf['tg.app_globals'].sa_engine = engine

        try:
            package_models = conf['package'].model
        except AttributeError:
            package_models = None

        model = conf.get('model', package_models)
        if model is None:
            raise TGConfigError('SQLAlchemy enabled, but no models provided')

        sqla_session = model.init_model(engine)
        if sqla_session is not None:
            # If init_model returns a specific session, keep it around
            # as the SQLAlchemy Session.
            conf['SQLASession'] = sqla_session

        if 'DBSession' not in conf:
            # If the user hasn't specified a default session, assume
            # he/she uses the default DBSession in model
            conf['DBSession'] = model.DBSession

    def _setup_auth(self, conf):
        """
        Override this method to define how you would like the authentication options
        to be setup for your application.
        """
        if hasattr(self, 'setup_sa_auth_backend'):
            warnings.warn("setup_sa_auth_backend is deprecated, please override"
                          "AppConfig._setup_auth instead", DeprecationWarning)
            return self.setup_sa_auth_backend()

        if conf['auth_backend'] in ("ming", "sqlalchemy"):
            sa_auth_conf = conf['sa_auth']
            if 'cookie_secret' not in sa_auth_conf and 'session.secret' not in conf:
                raise TGConfigError("You must provide a value for authentication cookies secret. "
                                    "Make sure that you have one of those options in app_cfg.py: "
                                    "'sa_auth.cookie_secret' or 'session.secret'")

            # The developer must have defined a 'sa_auth' section, because
            # values such as the User, Group or Permission classes must be
            # explicitly defined.
            sa_auth_conf.setdefault('form_plugin', None)
            sa_auth_conf.setdefault(
                'cookie_secret',
                conf.get('session.secret', conf.get('beaker.session.secret'))
            )

    def _setup_controller_wrappers(self, conf):
        # This trashes away the current config['controller_caller']
        # so that the call is idempotent.
        base_controller_caller = call_controller

        controller_caller = base_controller_caller
        for wrapper in conf.get('controller_wrappers', []):
            try:
                controller_caller = wrapper(controller_caller)
            except TypeError:
                controller_caller = _DeprecatedControllerWrapper(wrapper, conf, controller_caller)

        conf['controller_caller'] = controller_caller

    def _setup_renderers(self, conf):
        renderers = conf['renderers']
        rendering_engines = conf['rendering_engines']

        for renderer in renderers[:]:
            setup = getattr(self, 'setup_%s_renderer'%renderer, None)
            if setup is not None:
                # Backward compatible old-way of configuring rendering engines
                warnings.warn("Using setup_NAME_renderer to configure rendering engines"
                              "is now deprecated, please use register_rendering_engine "
                              "with a tg.renderers.base.RendererFactory subclass instead",
                              DeprecationWarning, stacklevel=2)

                success = setup()
                if success is False:
                    log.error('Failed to initialize %s template engine, removing it...' % renderer)
                    renderers.remove(renderer)
            elif renderer in rendering_engines:
                rendering_engine = rendering_engines[renderer]
                engines = rendering_engine.create(conf, conf['tg.app_globals'])
                if engines is None:
                    log.error('Failed to initialize %s template engine, removing it...' % renderer)
                    renderers.remove(renderer)
                else:
                    conf['render_functions'].update(engines)
            else:
                raise TGConfigError('This configuration object does '
                                    'not support the %s renderer' % renderer)

        milestones.renderers_ready.reach()

    def make_load_environment(self):
        """Return a load_environment function.

        The returned load_environment function can be called to configure
        the TurboGears runtime environment for this particular application.
        You can do this dynamically with multiple nested TG applications
        if necessary.

        """

        def load_environment(global_conf, app_conf):
            """Configure the TurboGears environment via ``tg.configuration.config``."""
            global_conf = Bunch(global_conf)
            app_conf = Bunch(app_conf)

            app_config = self._init_config(global_conf, app_conf)
            tg.hooks.notify('initialized_config', args=(self, app_config))
            tg.hooks.notify('startup', trap_exceptions=True)

            self._setup_helpers_and_globals(app_config)
            self._setup_auth(app_config)
            self._setup_renderers(app_config)
            self._setup_persistence(app_config)

            # Trigger milestone here so that it gets triggered even when
            # websetup (setup-app command) is performed.
            milestones.environment_loaded.reach()

            return app_config

        return load_environment

    def _add_error_middleware(self, app_config, app):
        """Add middleware which handles errors and exceptions."""
        from tg.error import ErrorReporter
        app = ErrorReporter(app, app_config, **app_config['tg.errorware'])

        if app_config['status_code_redirect'] is True:
            warnings.warn("Support for StatusCodeRedirect is deprecated and "
                          "will be removed in next major release", DeprecationWarning)

            if app_config['handle_error_page']:
                from tg.support.middlewares import StatusCodeRedirect
                # Display error documents for self.handle_status_codes status codes (and
                # 500 when debug is disabled)
                handled_status_codes = app_config['handle_status_codes']
                if not asbool(app_config['debug']):
                    handled_status_codes += [500]
                app = StatusCodeRedirect(app, handled_status_codes)

        return app

    def _add_slowreqs_middleware(self, app_config, app):
        from tg.error import SlowReqsReporter
        return SlowReqsReporter(app, app_config, **app_config['tg.slowreqs'])

    def _add_debugger_middleware(self, app_config, app):
        from tg.error import ErrorHandler
        return ErrorHandler(app, app_config)

    def _add_auth_middleware(self, app_config, app):
        """
        Configure authentication and authorization.
        """
        # Start with the current configured authentication options.
        # Depending on the auth backend a new auth_args dictionary
        # can replace this one later on.
        skip_authentication = asbool(app_config.get('skip_authentication', False))
        auth_args = copy(app_config['sa_auth'])

        # Configuring auth logging:
        if 'log_stream' not in auth_args:
            auth_args['log_stream'] = logging.getLogger('auth')

        # Removing keywords not used by repoze.who:
        auth_args.pop('password_encryption_method', None)

        if not skip_authentication and 'cookie_secret' not in auth_args:
            raise TGConfigError("base_config.sa_auth.cookie_secret is required "
                                "you must define it in app_cfg.py or set "
                                "sa_auth.cookie_secret in development.ini")

        if 'authmetadata' not in auth_args:  # pragma: no cover
            warnings.warn("Authentication configured without authmetadata, "
                          "this is not supported anymore and will be removed in future versions",
                          DeprecationWarning)

            # authmetadata not provided, fallback to old authentication setup
            if self.auth_backend == "sqlalchemy":
                from repoze.what.plugins.quickstart import setup_sql_auth
                app = setup_sql_auth(app, skip_authentication=skip_authentication, **auth_args)
            elif self.auth_backend == "ming":
                from tgming import setup_ming_auth
                app = setup_ming_auth(app, skip_authentication=skip_authentication, **auth_args)
        else:
            # Removing authmetadata as is not used by repoze.who:
            tgauthmetadata = auth_args.pop('authmetadata', None)

            try:
                pos = auth_args['authenticators'].index(('default', None))
            except KeyError:
                # Didn't specify authenticators, setup default one
                pos = None
            except ValueError:
                # Specified authenticators and default is not in there
                # so we want to skip default TG auth configuration.
                pos = -1

            if pos is None or pos >= 0:
                if getattr(tgauthmetadata, 'authenticate', None) is not None:
                    from tg.configuration.auth import create_default_authenticator
                    auth_args, tgauth = create_default_authenticator(tgauthmetadata, **auth_args)
                    authenticator = ('tgappauth', tgauth)
                elif self.auth_backend == "sqlalchemy":
                    from tg.configuration.sqla.auth import create_default_authenticator
                    auth_args, sqlauth = create_default_authenticator(**auth_args)
                    authenticator = ('sqlauth', sqlauth)
                elif self.auth_backend == "ming":
                    from tg.configuration.mongo.auth import create_default_authenticator
                    auth_args, mingauth = create_default_authenticator(**auth_args)
                    authenticator = ('mingauth', mingauth)
                else:
                    authenticator = None

                if authenticator is not None:
                    if pos is None:
                        auth_args['authenticators'] = [authenticator]
                    else:
                        # We make a copy so that we don't modify the original one.
                        auth_args['authenticators'] = copy(auth_args['authenticators'])
                        auth_args['authenticators'][pos] = authenticator

            from tg.configuration.auth import setup_auth
            app = setup_auth(app, skip_authentication=skip_authentication, **auth_args)

        return app

    def _add_core_middleware(self, conf, app):
        """Add support for sessions, and caching middlewares

        Those are all deprecated middlewares and will be removed in future TurboGears
        versions as they have been replaced by other tools.
        """
        if conf.get('use_session_middleware', False):
            warnings.warn('SessionMiddleware is deprecated and will be removed soon, '
                          'please consider using SessionApplicationWrapper instead.',
                          DeprecationWarning)
            from tg.support.middlewares import SessionMiddleware, BeakerSessionMiddleware
            if BeakerSessionMiddleware is object:  # pragma: no cover
                raise ImportError('Beaker not installed')
            app = SessionMiddleware(app, conf)

        if conf.get('use_cache_middleware', False):
            warnings.warn('CacheMiddleware is deprecated and will be removed soon, '
                          'please consider using CacheApplicationWrapper instead.',
                          DeprecationWarning)
            from tg.support.middlewares import CacheMiddleware, BeakerCacheMiddleware
            if BeakerCacheMiddleware is object:  # pragma: no cover
                raise ImportError('Beaker not installed')
            app = CacheMiddleware(app, conf)

        return app

    def _add_tosca_middleware(self, conf, app):
        """Configure the ToscaWidgets middleware"""
        import tw
        from tw.api import make_middleware as tw_middleware
        from tg.i18n import ugettext

        twconfig = {'toscawidgets.framework.default_view': conf['default_renderer'],
                    'toscawidgets.framework.translator': ugettext,
                    'toscawidgets.middleware.inject_resources': True,
                    }
        for k,v in conf.items():
            if k.startswith('toscawidgets.framework.') or k.startswith('toscawidgets.middleware.'):
                twconfig[k] = v

        resource_variant = conf.get('toscawidgets.framework.resource_variant')
        if resource_variant is not None:
            import tw.api
            tw.api.resources.registry.ACTIVE_VARIANT = resource_variant
            # remove it from the middleware madness
            del twconfig['toscawidgets.framework.resource_variant']

        app = tw_middleware(app, twconfig)

        if conf['default_renderer'] in ('genshi','mako'):
            tw.framework.default_view = conf['default_renderer']

        return app

    def _add_tosca2_middleware(self, conf, app):
        """Configure the ToscaWidgets2 middleware"""
        from tw2.core.middleware import Config, TwMiddleware
        from tg.i18n import ugettext, get_lang

        available_renderers = set(conf['renderers'])
        shared_engines = list(available_renderers & set(Config.preferred_rendering_engines))
        if not shared_engines:
            raise TGConfigError('None of the configured rendering engines is supported'
                                'by ToscaWidgets2, unable to configure ToscaWidgets.')

        default_renderer = conf['default_renderer']
        if default_renderer in shared_engines:
            tw2_engines = [default_renderer] + shared_engines
            tw2_default_engine = default_renderer
        else:
            # If preferred rendering engine is not available in TW2, fallback to another one
            tw2_engines = shared_engines
            tw2_default_engine = shared_engines[0]

        default_tw2_config = dict(default_engine=tw2_default_engine,
                                  preferred_rendering_engines=tw2_engines,
                                  translator=ugettext,
                                  get_lang=lambda: get_lang(all=False),
                                  auto_reload_templates=conf['auto_reload_templates'],
                                  controller_prefix='/tw2/controllers/',
                                  res_prefix='/tw2/resources/',
                                  debug=conf['debug'],
                                  rendering_extension_lookup={
                                       'mako': ['mak', 'mako'],
                                       'genshi': ['genshi', 'html'],
                                       'jinja': ['jinja', 'jinja2'],
                                       'kajiki': ['kajiki', 'xhtml', 'xml']
                                  })

        default_tw2_config.update(conf.get('custom_tw2_config', {}))
        app = TwMiddleware(app, **default_tw2_config)
        return app

    def _add_static_file_middleware(self, conf, app):
        app = StaticsMiddleware(app, conf['paths']['static_files'])
        return app

    def _add_tm_middleware(self, conf, app):
        """Set up the transaction management middleware.

        To abort a transaction inside a TG2 app::

          import transaction
          transaction.doom()

        By default http error responses also roll back transactions, but this
        behavior can be overridden by overriding base_config['tm.commit_veto'].

        """
        warnings.warn("transaction manager middleware has been replaced by "
                      "TransactionApplicationWrapper", DeprecationWarning, stacklevel=2)
        from tg.support.transaction_manager import TGTransactionManager

        # TODO: remove self.commit_veto option in future release
        # backward compatibility with "commit_veto" option
        if 'commit_veto' in conf:
            conf['tm.commit_veto'] = conf['commit_veto']
            warnings.warn("commit_veto option has been replaced by tm.commit_veto",
                          DeprecationWarning, stacklevel=2)

        return TGTransactionManager(app, conf)

    def _add_ming_middleware(self, conf, app):
        """Set up the ming middleware for the unit of work"""
        from tg.support.middlewares import MingSessionRemoverMiddleware
        from ming.odm import ThreadLocalODMSession
        return MingSessionRemoverMiddleware(ThreadLocalODMSession, app)

    def _add_sqlalchemy_middleware(self, conf, app):
        """Set up middleware that cleans up the sqlalchemy session.

        The default behavior of TG 2 is to clean up the session on every
        request.  Only override this method if you know what you are doing!

        """
        dbsession = conf.get('SQLASession')
        if dbsession is None:
            dbsession = conf['DBSession']
        return DBSessionRemoverMiddleware(dbsession, app)

    def _add_seekable_body_middleware(self, conf, app):
        """Make the request body seekable, so it can be read multiple times."""
        return SeekableRequestBodyMiddleware(app)

    def setup_tg_wsgi_app(self, load_environment=None):
        """Create a base TG app, with all the standard middleware.

        ``load_environment``
            A required callable, which sets up the basic evironment
            needed for the application.
        ``setup_vars``
            A dictionary with all special values necessary for setting up
            the base wsgi app.

        """

        def make_base_app(global_conf=None, wrap_app=None, full_stack=False, **app_conf):
            """Create a tg WSGI application and return it.

            ``wrap_app``
                a WSGI middleware component which takes the core turbogears
                application and wraps it -- inside all the WSGI-components
                provided by TG and Pylons. This allows you to work with the
                full environment that your TG application would get before
                anything happens in the application itself.

            ``global_conf``
                The inherited configuration for this application. Normally
                from the [DEFAULT] section of the Paste ini file.

            ``full_stack``
                Whether or not this application provides a full WSGI stack (by
                default, meaning it handles its own exceptions and errors).
                Disable full_stack when this application is "managed" by
                another WSGI middleware.

            ``app_conf``
                The application's local configuration. Normally specified in
                the [app:<name>] section of the Paste ini file (where <name>
                defaults to main).

            """
            from tg import TGApp

            if global_conf is None:
                global_conf = {}

            # Configure the Application environment
            if load_environment:
                app_config = load_environment(global_conf, app_conf)
            else:
                app_config = tg.config._current_obj()

                # In case load_environment was not performed we manually trigger all
                # the milestones to ensure that events related to configuration milestones
                # are performed in any case.
                milestones.config_ready.reach()
                milestones.renderers_ready.reach()
                milestones.environment_loaded.reach()

            # Apply controller wrappers to controller caller
            self._setup_controller_wrappers(app_config)

            app = TGApp(app_config)

            tg.hooks.notify('configure_new_app', args=(app,))

            if wrap_app:
                app = wrap_app(app)

            app = tg.hooks.notify_with_value('before_config', app)

            app = self._add_core_middleware(app_config, app)

            if app_config.get('auth_backend', False):
                app = self._add_auth_middleware(app_config, app)

            if app_config.get('use_transaction_manager', False):
                app = self._add_tm_middleware(app_config, app)

            # TODO: Middlewares before this point should be converted to App Wrappers.
            # They provide some basic TG features like AUTH, Caching and transactions
            # which should be app wrappers to make possible to add wrappers in the
            # stack before or after them.

            if app_config.get('use_toscawidgets', False):
                app = self._add_tosca_middleware(app_config, app)

            if app_config.get('use_toscawidgets2', False):
                app = self._add_tosca2_middleware(app_config, app)

            # from here on, due to TW2, the response is a generator
            # so any middleware that relies on the response to be
            # a string needs to be applied before this point.

            if app_config.get('use_sqlalchemy', False):
                app = self._add_sqlalchemy_middleware(app_config, app)

            if app_config.get('use_ming', False):
                app = self._add_ming_middleware(app_config, app)

            if app_config.get('make_body_seekable', False):
                app = self._add_seekable_body_middleware(app_config, app)

            if asbool(full_stack):
                # This should never be true for internal nested apps
                app = self._add_slowreqs_middleware(app_config, app)
                app = self._add_error_middleware(app_config, app)

            # Establish the registry for this application
            app = RegistryManager(app, streaming=asbool(app_config.get('registry_streaming', True)),
                                  preserve_exceptions=asbool(app_config.get('debug')))

            # Place the debuggers after the registry so that we
            # can preserve context in case of exceptions
            app = self._add_debugger_middleware(app_config, app)

            # Static files (if running in production, and Apache or another
            # web server is serving static files)
            if app_config.get('serve_static', False):
                app = self._add_static_file_middleware(app_config, app)

            app = tg.hooks.notify_with_value('after_config', app)

            return app

        return make_base_app

    def make_wsgi_app(self, **app_conf):
        loadenv = self.make_load_environment()
        return self.setup_tg_wsgi_app(loadenv)(**app_conf)



from tg import jsonify, lurl
from datetime import datetime
from decimal import Decimal
from nose.tools import raises
from nose import SkipTest
from webob.multidict import MultiDict
import json
from tg.util import LazyString
from tg.util.webtest import test_context


class Foo(object):
    def __init__(self, bar):
        self.bar = bar

class Bar(object):
    def __init__(self, bar):
        self.bar = bar
    def __json__(self):
        return 'bar-%s' % self.bar

class Baz(object):
    pass

def test_string():
    d = "string"
    encoded = jsonify.encode(d)
    assert encoded == '"string"'

@raises(jsonify.JsonEncodeError)
def test_list():
    d = ['a', 1, 'b', 2]
    encoded = jsonify.encode(d)
    assert encoded == '["a", 1, "b", 2]'

def test_list_allowed_iter():
    lists_encoder = jsonify.JSONEncoder(allow_lists=True)
    d = ['a', 1, 'b', 2]
    encoded = jsonify.encode(d, lists_encoder)
    assert encoded == '["a", 1, "b", 2]'

@raises(jsonify.JsonEncodeError)
def test_list_iter():
    d = list(range(3))
    encoded = jsonify.encode_iter(d)
    assert ''.join(jsonify.encode_iter(d)) == jsonify.encode(d)

def test_list_allowed_iter():
    lists_encoder = jsonify.JSONEncoder(allow_lists=True)
    d = list(range(3))
    encoded = jsonify.encode_iter(d, lists_encoder)
    assert ''.join(encoded) == '[0, 1, 2]'

def test_dictionary():
    d = {'a': 1, 'b': 2}
    encoded = jsonify.encode(d)
    expected = json.dumps(json.loads('{"a": 1, "b": 2}'))
    assert encoded == expected

@raises(jsonify.JsonEncodeError)
def test_nospecificjson():
    b = Baz()
    try:
        encoded = jsonify.encode(b)
    except TypeError as e:
        pass
    assert  "is not JSON serializable" in e.message 

def test_exlicitjson():
    b = Bar("bq")
    encoded = jsonify.encode(b)
    assert encoded == '"bar-bq"'

@raises(jsonify.JsonEncodeError)
def test_exlicitjson_in_list():
    b = Bar("bq")
    d = [b]
    encoded = jsonify.encode(d)
    assert encoded == '["bar-bq"]'

def test_exlicitjson_in_dict():
    b = Bar("bq")
    d = {"b": b}
    encoded = jsonify.encode(d)
    assert encoded == '{"b": "bar-bq"}'

def test_datetime():
    d = datetime.utcnow()
    encoded = jsonify.encode({'date':d})
    assert str(d.year) in encoded, (str(d), encoded)

def test_datetime_iso():
    isodates_encoder = jsonify.JSONEncoder(isodates=True)

    d = datetime.utcnow()
    encoded = jsonify.encode({'date': d}, encoder=isodates_encoder)

    isoformat_without_millis = json.dumps({'date': d.isoformat()[:19]})
    assert isoformat_without_millis == encoded, (isoformat_without_millis, encoded)
    assert 'T' in encoded, encoded

def test_date_iso():
    isodates_encoder = jsonify.JSONEncoder(isodates=True)

    d = datetime.utcnow().date()
    encoded = jsonify.encode({'date': d}, encoder=isodates_encoder)

    isoformat_without_millis = json.dumps({'date': d.isoformat()})
    assert isoformat_without_millis == encoded, (isoformat_without_millis, encoded)

    loaded_date = json.loads(encoded)
    assert len(loaded_date['date'].split('-')) == 3

def test_datetime_time():
    d = datetime.utcnow().time()
    encoded = jsonify.encode({'date':d})
    assert str(d.hour) in encoded, (str(d), encoded)

def test_datetime_time_iso():
    isodates_encoder = jsonify.JSONEncoder(isodates=True)

    d = datetime.utcnow().time()
    encoded = jsonify.encode({'date': d}, encoder=isodates_encoder)

    isoformat_without_millis = json.dumps({'date': d.isoformat()[:8]})
    assert isoformat_without_millis == encoded, (isoformat_without_millis, encoded)

def test_decimal():
    d = Decimal('3.14')
    encoded = jsonify.encode({'dec':d})
    assert '3.14' in encoded

def test_objectid():
    try:
        from bson import ObjectId
    except:
        raise SkipTest()

    d = ObjectId('507f1f77bcf86cd799439011')
    encoded = jsonify.encode({'oid':d})
    assert encoded == '{"oid": "%s"}' % d, encoded

def test_multidict():
    d = MultiDict({'v':1})
    encoded = jsonify.encode({'md':d})
    assert encoded == '{"md": {"v": 1}}', encoded

def test_json_encode_lazy_url():
    with test_context(None, '/'):
        url = lurl('/test')
        encoded = jsonify.encode({'url': url})
        assert encoded == '{"url": "/test"}', encoded

def test_json_encode_lazy_string():
    text = LazyString(lambda: 'TEST_STRING')
    encoded = jsonify.encode({'text': text})
    assert encoded == '{"text": "TEST_STRING"}', encoded

def test_json_encode_generators():
    encoded = jsonify.encode({'values': (v for v in [1, 2, 3])})
    assert encoded == '{"values": [1, 2, 3]}', encoded


from __future__ import absolute_import

import os
import logging
import stat
from tg.configuration.utils import coerce_config

try:
    import threading
except ImportError: #pragma: no cover
    import dummy_threading as threading

from tg.support.converters import asbool
from markupsafe import Markup
from tg.render import cached_template
from .base import RendererFactory

try:
    import mako
except ImportError:  # pragma: no cover
    mako = None

if mako is not None:
    from mako.template import Template
    from mako import exceptions
    from mako.lookup import TemplateLookup

__all__ = ['MakoRenderer']

log = logging.getLogger(__name__)


class MakoRenderer(RendererFactory):
    """
    Configuration Options available as ``templating.mako.*``:

        - ``templating.mako.template_extension`` -> Mako Templates extension, default ``.mak``
        - ``templating.mako.compiled_templates_dir`` -> Where to store mako precompiled templates.
          By default templates are only stored in memory and not on disk.
    """
    #: Configuration Options that can be set as ``templating.mako.*``.
    CONFIG_OPTIONS = {
        'compiled_templates_dir': str,
        'template_extension': str
    }
    engines = {'mako': {'content_type': 'text/html'}}

    @classmethod
    def create(cls, config, app_globals):
        """
        Setup a renderer and loader for mako templates.
        """
        if mako is None:  # pragma: no cover
            return None

        use_dotted_templatenames = config.get('use_dotted_templatenames', True)

        options = coerce_config(config, 'templating.mako.', cls.CONFIG_OPTIONS)

        # If no dotted names support was required we will just setup
        # a file system based template lookup mechanism.
        compiled_dir = options.get('compiled_templates_dir', None)

        if not compiled_dir or compiled_dir.lower() in ('none', 'false'):
            # Cache compiled templates in-memory
            compiled_dir = None
        else:
            bad_path = None
            if os.path.exists(compiled_dir):
                if not os.access(compiled_dir, os.W_OK):
                    bad_path = compiled_dir
                    compiled_dir = None
            else:
                try:
                    os.makedirs(compiled_dir)
                except:
                    bad_path = compiled_dir
                    compiled_dir = None
            if bad_path:
                log.warn("Unable to write cached templates to %r; falling back "
                         "to an in-memory cache. Please set the `templating.mak"
                         "o.compiled_templates_dir` configuration option to a "
                         "writable directory." % bad_path)

        template_extension = options.get('template_extension', '.mak')

        # Support dotted names by using a slightly different template
        # lookup system that will return templates from dotted template notation.
        dotted_loader = DottedTemplateLookup(
            input_encoding='utf-8', output_encoding='utf-8',
            imports=['from markupsafe import escape_silent as escape'],
            package_name=config['package_name'],
            find_template_file=lambda t: app_globals.dotted_filename_finder.get_dotted_filename(
                t, template_extension=template_extension
            ),
            template_extension=template_extension,
            module_directory=compiled_dir,
            default_filters=['escape'],
            auto_reload_templates=config['auto_reload_templates'])

        normal_loader = TemplateLookup(
            directories=config['paths']['templates'],
            module_directory=compiled_dir,
            input_encoding='utf-8', output_encoding='utf-8',
            imports=['from markupsafe import escape_silent as escape'],
            default_filters=['escape'],
            filesystem_checks=config['auto_reload_templates'])

        return {'mako': cls(use_dotted_templatenames, template_extension,
                            dotted_loader, normal_loader)}

    def __init__(self, use_dotted_templatenames, template_extension,
                 dotted_loader, normal_loader):
        self.dotted_loader = dotted_loader
        self.normal_loader = normal_loader
        self.use_dotted_templatenames = use_dotted_templatenames
        self.template_extension = template_extension

    def __call__(self, template_name, template_vars,
                 cache_key=None, cache_type=None, cache_expire=None):

        if self.use_dotted_templatenames and not template_name.endswith(self.template_extension):
            template_name = self.dotted_loader.find_template_file(template_name)
            loader = self.dotted_loader
        else:
            loader = self.normal_loader

        # Create a render callable for the cache function
        def render_template():
            # Grab a template reference
            template = loader.get_template(template_name)
            return Markup(template.render_unicode(**template_vars))

        return cached_template(template_name, render_template, cache_key=cache_key,
                               cache_type=cache_type, cache_expire=cache_expire)


class DottedTemplateLookup(object):
    """Mako template lookup emulation that supports
    zipped applications and dotted filenames.

    This is an emulation of the Mako template lookup that will handle
    get_template and support dotted names in Python path notation
    to support zipped eggs.

    This is necessary because Mako asserts that your project will always
    be installed in a zip-unsafe manner with all files somewhere on the
    hard drive.

    This is not the case when you want your application to be deployed
    in a single zip file (zip-safe). If you want to deploy in a zip
    file _and_ use the dotted template name notation then this class
    is necessary because it emulates files on the filesystem for the
    underlying Mako engine while they are in fact in your zip file.

    """

    def __init__(self, input_encoding, output_encoding,
                 imports, default_filters, package_name,
                 find_template_file, template_extension='.mak',
                 module_directory=None,
                 auto_reload_templates=False):

        self.package_name = package_name
        self.find_template_file = find_template_file

        self.input_encoding = input_encoding
        self.output_encoding = output_encoding
        self.imports = imports
        self.default_filters = default_filters
        # implement a cache for the loaded templates
        self.template_cache = dict()
        # implement a cache for the filename lookups
        self.template_filenames_cache = dict()
        self.module_directory = module_directory
        self.auto_reload = auto_reload_templates
        self.template_extension = template_extension

        # a mutex to ensure thread safeness during template loading
        self._mutex = threading.Lock()

    def adjust_uri(self, uri, relativeto):
        """Adjust the given uri relative to a filename.

        This method is used by mako for filesystem based reasons.
        In dotted lookup land we don't adjust uri so we just return
        the value we are given without any change.

        """
        if uri.startswith('local:'):
            uri = self.package_name + '.' + uri[6:]

        if '.' in uri and not uri.endswith(self.template_extension):
            # We are in the DottedTemplateLookup system so dots in
            # names should be treated as a Python path. Since this
            # method is called by template inheritance we must
            # support dotted names also in the inheritance.
            result = self.find_template_file(uri)

            if not uri in self.template_filenames_cache:
                # feed our filename cache if needed.
                self.template_filenames_cache[uri] = result

        else:
            # no dot detected, just return plain name
            result = uri

        return result

    def __check(self, template):
        """private method used to verify if a template has changed
        since the last time it has been put in cache...

        This method being based on the mtime of a real file this should
        never be called on a zipped deployed application.

        This method is a ~copy/paste of the original caching system from
        the Mako lookup loader.

        """
        if template.filename is None:
            return template


        if not os.path.exists(template.filename):
            # remove from cache.
            self.template_cache.pop(template.filename, None)
            raise exceptions.TemplateLookupException(
                    "Cant locate template '%s'" % template.filename)

        elif template.module._modified_time < os.stat(
                template.filename)[stat.ST_MTIME]:

            # cache is too old, remove old template
            # from cache and reload.
            self.template_cache.pop(template.filename, None)
            return self.__load(template.filename)

        else:
            # cache is correct, use it.
            return template

    def __load(self, filename):
        """real loader function. copy paste from the mako template
        loader.

        """
        # make sure the template loading from filesystem is only done
        # one thread at a time to avoid bad clashes...
        self._mutex.acquire()
        try:
            try:
                # try returning from cache one more time in case
                # concurrent thread already loaded
                return self.template_cache[filename]

            except KeyError:
                # not in cache yet... we can continue normally
                pass

            try:
                self.template_cache[filename] = Template(
                    filename=filename,
                    module_directory=self.module_directory,
                    input_encoding=self.input_encoding,
                    output_encoding=self.output_encoding,
                    default_filters=self.default_filters,
                    imports=self.imports,
                    lookup=self)

                return self.template_cache[filename]

            except:
                self.template_cache.pop(filename, None)
                raise

        finally:
            # _always_ release the lock once done to avoid
            # "thread lock" effect
            self._mutex.release()

    def get_template(self, template_name):
        """this is the emulated method that must return a template
        instance based on a given template name
        """

        if template_name not in self.template_cache:
            # the template string is not yet loaded into the cache.
            # Do so now
            self.__load(template_name)

        if self.auto_reload:
            # AUTO RELOADING will be activated only if user has
            # explicitly asked for it in the configuration
            # return the template, but first make sure it's not outdated
            # and if outdated, refresh the cache.
            return self.__check(self.template_cache[template_name])

        else:
            return self.template_cache[template_name]




from tests.test_stack import TestConfig, app_from_config

def setup_noDB():
    base_config = TestConfig(folder = 'config',
                             values = {'use_sqlalchemy': False,
                                       'use_toscawidgets': False,
                                       'use_toscawidgets2':False}
                             )
    return app_from_config(base_config)

def test_basic_stack():
    app = setup_noDB()
    resp = app.get('/')
    assert resp.body.decode('ascii') == "my foo"

def test_config_reading():
    """Ensure that the config object can be read via dict and attr access"""
    app = setup_noDB()
    resp = app.get('/config_test')
    resp_body = resp.body.decode('ascii')

    assert "default_renderer" in resp_body
    resp = app.get('/config_attr_lookup')
    assert "genshi" in resp_body
    resp = app.get('/config_dotted_values')
    assert "root" in resp_body

def test_config_writing():
    """Ensure that new values can be added to the config object"""
    app = setup_noDB()
    value = "gooberblue"
    resp = app.get('/config_attr_set/'+value)
    resp_body = resp.body.decode('ascii')

    assert value in resp_body
    resp = app.get('/config_dict_set/'+value)
    assert value in resp_body




"""Main Controller"""
import tg
from tg import expose, redirect, config, validate, override_template, response, render_template, tmpl_context
from tg import cache, i18n, request
from tg.decorators import paginate, use_custom_format, with_trailing_slash, Decoration, before_render, decode_params
from tg.controllers import TGController
from tg.validation import TGValidationError
from tg._compat import PY3
from tg.render import _get_tg_vars, cached_template
import datetime

from webob.exc import HTTPForbidden

if not PY3:
    from tw.forms import TableForm, TextField, CalendarDatePicker, SingleSelectField, TextArea
    from tw.api import WidgetsList

    class MovieForm(TableForm):
        # This WidgetsList is just a container
        class fields(WidgetsList):
            title = TextField()
            year = TextField(size=4, default=1984)
            description = TextArea()

    #then, we create an instance of this form
    base_movie_form = MovieForm("movie_form", action='create')
else:
    base_movie_form = None


import tw2.forms as tw2f
import tw2.core as tw2c
class TW2MovieForm(tw2f.TableForm):
    title = tw2f.TextField(validator=tw2c.Required)
    year = tw2f.TextField(size=4, validator=tw2c.IntValidator)

tw2_movie_form = TW2MovieForm()


class IntValidator(object):
    def to_python(self, value):
        try:
            return int(value)
        except:
            raise TGValidationError('Not a number')


class GoodJsonObject(object):

    def __json__(self):
        return {'Json':'Rocks'}

class BadJsonObject(object):
    pass


class JsonController(TGController):

    @expose('json')
    def json(self):
        return dict(a='hello world', b=True)

    @expose('json', exclude_names=["b"])
    def excluded_b(self):
        return dict(a="visible", b="invisible")

    @expose('json')
    @expose('genshi:test', content_type='application/xml')
    def xml_or_json(self):
        return dict(name="John Carter", title='officer', status='missing')

    @expose('json')
    def json_with_object(self):
        return dict(obj=GoodJsonObject())

    @expose('json')
    def json_with_bad_object(self):
        return dict(obj=BadJsonObject())


class SubClassableController(TGController):
    @expose('genshi:index.html')
    def index(self):
        return {}

    @expose('genshi:index.html')
    def index_override(self):
        return {}

    def before_render_data(remainder, params, output):
        output['parent_value'] = 'PARENT'

    @expose('json')
    @before_render(before_render_data)
    def data(self):
        return {'v':5}

class SubClassingController(SubClassableController):
    @expose(inherit=True)
    def index(self, *args, **kw):
        return super(SubClassingController, self).index(*args, **kw)

    @expose('genshi:genshi_doctype.html', inherit=True)
    def index_override(self, *args, **kw):
        return super(SubClassingController, self).index_override(*args, **kw)

    def before_render_data(remainder, params, output):
        output['child_value'] = 'CHILD'

    @expose(inherit=True)
    @before_render(before_render_data)
    def data(self, *args, **kw):
        return super(SubClassingController, self).data(*args, **kw)


class ErrorController(TGController):
    @expose('genshi:index.html')
    def document(self, *args, **kwargs):
        return dict()


class RootController(TGController):

    j = JsonController()
    sub1 = SubClassableController()
    sub2 = SubClassingController()
    error = ErrorController()

    @expose('genshi:index.html')
    def index(self):
        return {}

    @expose('json')
    def aborted_json(self):
        raise HTTPForbidden(json_body={'error': 'value'})

    @expose('kajiki:tests.test_stack.rendering.templates.index')
    @expose('json')
    def according_to_content_type(self, ctype):
        tg.response.content_type = ctype
        return dict(value='SomeValue')

    @expose('genshi:genshi_doctype.html')
    def auto_doctype(self):
        return {}

    @expose('genshi:genshi_doctype.html', content_type='text/html')
    def auto_doctype_html(self):
        return {}

    @expose('genshi:genshi_doctype.html', content_type='application/xhtml+xml')
    def auto_doctype_xhtml(self):
        return {}

    @expose('genshi:genshi_doctype.html', render_params=dict(doctype=None))
    def explicit_no_doctype(self):
        return {}

    @expose('genshi:genshi_doctype.html', render_params=dict(doctype='html'))
    def explicit_doctype_html(self):
        return {}

    @expose('genshi:genshi_doctype.html', render_params=dict(doctype='xhtml'))
    def explicit_doctype_xhtml(self):
        return {}

    @expose('genshi:genshi_form.html')
    def form(self):
        return dict(form=base_movie_form)

    @expose('genshi:genshi_form.html')
    def tw2form(self):
        return dict(form=tw2_movie_form)

    @expose('genshi:genshi_foreign.html')
    def foreign(self):
        return {}

    @expose('json')
    @validate(form=base_movie_form)
    def process_form_errors(self, **kwargs):
        #add error messages to the kwargs dictionary and return it
        kwargs['errors'] = request.validation['errors']
        return dict(kwargs)

    @expose()
    @paginate('testdata')
    def paginated_text(self):
        return '''Some Text'''

    @expose('kajiki:kajiki_paginated.xhtml')
    @expose('json')
    @paginate('testdata', max_items_per_page=20)
    def paginated(self, n):
        return dict(testdata=range(int(n)))

    @expose('kajiki:kajiki_paginated.xhtml')
    @paginate('testdata')
    def paginate_with_params(self, n):
        url_params = dict(param1='hi', param2='man')
        return dict(testdata=range(int(n)), url_params=url_params)

    @expose('kajiki:kajiki_paginated.xhtml')
    @paginate('testdata')
    @validate(dict(n=IntValidator()))
    def paginated_validated(self, n):
        return dict(testdata=range(n))

    @expose('kajiki:kajiki_paginated.xhtml')
    @validate(dict(n=IntValidator()))
    @paginate('testdata')
    def validated_paginated(self, n):
        return dict(testdata=range(n))

    @expose('kajiki:kajiki_paginated.xhtml')
    @paginate('testdata', use_prefix=True)
    @paginate('testdata2', use_prefix=True)
    def multiple_paginators(self, n):
        n = int(n)
        return dict(testdata=range(n), testdata2=range(n+100, n+100+n))

    @expose('genshi:genshi_inherits.html')
    def genshi_inherits(self):
        return {}

    @expose('genshi:genshi_inherits_sub.html')
    def genshi_inherits_sub(self):
        return {}

    @expose('genshi:sub/frombottom.html')
    def genshi_inherits_sub_from_bottom(self):
        return {}

    @expose('jinja:jinja_noop.jinja')
    def jinja_index(self):
        return {}

    @expose('jinja:jinja_autoload.jinja')
    def jinja_autoload(self):
        return {}

    @expose('jinja:jinja_inherits.jinja')
    def jinja_inherits(self):
        return {}

    @expose('jinja:tests.test_stack.rendering.templates.jinja_noop')
    def jinja_dotted(self):
        return {}

    @expose('jinja:tests.test_stack.rendering.templates.jinja_inherits_dotted')
    def jinja_inherits_dotted(self):
        return {}

    @expose('jinja:tests.test_stack.rendering.templates.jinja_inherits')
    def jinja_inherits_mixed(self):
        return {}

    @expose('jinja:jinja_extensions.jinja')
    def jinja_extensions(self):
        test_autoescape_on = "<b>Test Autoescape On</b>"
        test_autoescape_off = "<b>Autoescape Off</b>"
        return dict(test_autoescape_off=test_autoescape_off,
                test_autoescape_on=test_autoescape_on)

    @expose('jinja:jinja_filters.jinja')
    def jinja_filters(self):
        return {}

    @expose('jinja:jinja_buildins.jinja')
    def jinja_buildins(self):
        return {}

    @expose('jinja:jinja_i18n.jinja')
    def jinja_i18n(self):
        return {}

    @expose('jinja:jinja_i18n.jinja')
    def jinja_i18n_en(self):
        i18n.set_temporary_lang("en")
        return {}

    @expose('jinja:jinja_i18n.jinja')
    def jinja_i18n_de(self):
        i18n.set_temporary_lang("de")
        return {}

    @expose('chameleon_genshi:index.html')
    def chameleon_genshi_index(self):
        return {}

    @expose('chameleon_genshi:genshi_inherits.html')
    def chameleon_genshi_inherits(self):
        return {}

    @expose('mako:mako_noop.mak')
    def mako_index(self):
        return {}

    @expose('mako:mako_inherits.mak')
    def mako_inherits(self):
        return {}

    @expose('chameleon_genshi:tests.test_stack.rendering.templates.index')
    def chameleon_index_dotted(self):
        return {}

    @expose('kajiki:tests.test_stack.rendering.templates.index')
    def kajiki_index_dotted(self):
        return {}

    @expose('kajiki:tests.test_stack.rendering.templates.missing')
    def kajiki_missing_template(self):
        return {}

    @expose('kajiki:tests.test_stack.rendering.templates.kajiki_i18n')
    def kajiki_i18n(self):
        return {}

    @expose('kajiki:tests.test_stack.rendering.templates.kajiki_i18n')
    def kajiki_i18n_de(self):
        i18n.set_temporary_lang("de")
        return {}

    @expose('genshi:tests.test_stack.rendering.templates.index')
    def index_dotted(self):
        return {}

    @expose('kajiki:tests.test_stack.rendering.templates.index!html')
    def index_dotted_with_forced_extension(self):
        # This explicitly exposes a Genshi template through Kajiki
        # using the !ext syntax to force correct extension resolution.
        tmpl_context.now = lambda: 'IT WORKS'
        return {}

    @expose('genshi:tests.test_stack.rendering.templates.genshi_inherits')
    def genshi_inherits_dotted(self):
        return {}

    @expose('genshi:tests.test_stack.rendering.templates.genshi_inherits_sub_dotted')
    def genshi_inherits_sub_dotted(self):
        return {}

    @expose('genshi:tests.test_stack.rendering.templates.sub.frombottom_dotted')
    def genshi_inherits_sub_dotted_from_bottom(self):
        return {}

    @expose('mako:tests.test_stack.rendering.templates.mako_noop')
    def mako_index_dotted(self):
        return {}

    @expose('mako:tests.test_stack.rendering.templates.mako_inherits_dotted')
    def mako_inherits_dotted(self):
        return {}

    @expose('json')
    @expose('genshi:index.html')
    def html_and_json(self):
        return {}

    @expose('json', custom_format='json')
    @expose('mako:mako_custom_format.mak', content_type='text/xml', custom_format='xml')
    @expose('genshi:genshi_custom_format.html', content_type='text/html', custom_format='html')
    def custom_format(self, format='default'):
        if format != 'default':
            use_custom_format(self.custom_format, format)
            return dict(format=format, status="ok")
        else:
            return 'OK'

    @expose("genshi:tests.non_overridden")
    def template_override(self, override=False):
        if override:
            override_template(self.template_override, "genshi:tests.overridden")
        return dict()

    @with_trailing_slash
    @expose("genshi:tests.non_overridden")
    def template_override_wts(self, override=False):
        if override:
            override_template(self.template_override_wts, "genshi:tests.overridden")
        return dict()

    @expose(content_type='text/javascript')
    def template_override_content_type(self, override=False):
        if override:
            override_template(self.template_override_content_type, "mako:tests.overridden_js")
            return dict()
        else:
            return "alert('Not overridden')"

    @expose('mako:mako_custom_format.mak', content_type='text/xml')
    @expose('genshi:genshi_custom_format.html', content_type='text/html')
    def template_override_multiple_content_type(self, override=False):
        if override:
            override_template(self.template_override_multiple_content_type, "mako:mako_noop.mak")
        return dict(format='something', status="ok")

    @expose()
    def jinja2_manual_rendering(self, frompylons=False):
        try:
            import pylons
        except ImportError:
            frompylons = False

        if frompylons:
            from pylons.templating import render_jinja2
            return render_jinja2('jinja_inherits.jinja')
        else:
            return render_template({}, 'jinja', 'jinja_inherits.jinja')

    @expose()
    def no_template_generator(self):
        def output():
            num = 0
            while num < 5:
                num += 1
                yield str(num).encode('ascii')
        return output()
    
    @expose()
    def genshi_manual_rendering_with_doctype(self, doctype=None):
        response.content_type = 'text/html'
        response.charset = 'utf-8'
        return render_template({}, 'genshi', 'genshi_doctype.html', doctype=doctype)

    @expose('mako:mako_custom_format.mak')
    @expose('genshi:genshi_custom_format.html')
    def multiple_engines(self):
        deco = Decoration.get_decoration(self.multiple_engines)
        used_engine = deco.engines.get('text/html')[0]
        return dict(format=used_engine, status='ok')

    @expose('json')
    def get_tg_vars(self):
        return dict(tg_vars=list(_get_tg_vars().keys()))

    @expose('genshi:index.html')
    def template_caching(self):
        from datetime import datetime
        tmpl_context.now = datetime.utcnow
        return dict(tg_cache={'key':'TEMPLATE_CACHE_TEST',
                              'type':'memory',
                              'expire':'never'})

    @expose('genshi:index.html')
    def template_caching_default_type(self):
        from datetime import datetime
        tmpl_context.now = datetime.utcnow
        return dict(tg_cache={'key':'TEMPLATE_CACHE_TEST2',
                              'expire':'never'})

    @expose('json')
    def template_caching_options(self, **kwargs):
        _cache_options = {}
        class FakeCache(object):
            def get_cache(self, *args, **kwargs):
                _cache_options['args'] = args
                _cache_options['kwargs'] = kwargs
                try:
                    c = cache.get_cache(*args, **kwargs)
                    _cache_options['cls'] = c.namespace.__class__.__name__
                except TypeError:
                    _cache_options['cls'] = 'NoImplementation'
                    c = cache.get_cache(*args, type='memory', **kwargs)
                return c

        tg.cache.kwargs['type'] = 'NoImplementation'
        old_cache = tg.cache
        tg.cache = FakeCache()

        try:
            def render_func(*args, **kw):
                return 'OK'
            cached_template('index.html', render_func, **kwargs)
            return _cache_options
        finally:
            tg.cache = old_cache

    @expose('jsonp', render_params={'callback_param': 'call'})
    def get_jsonp(self, **kwargs):
        return {'value': 5}

    @expose('jsonp', render_params={'callback_param': 'call', 'key': 'result'})
    def get_jsonp_with_key(self, **kwargs):
        return {'result': {'value': 5}}

    @expose('json')
    def get_json_isodates_default(self, **kwargs):
        return {'date': datetime.datetime.utcnow()}

    @expose('json', render_params={'isodates': True})
    def get_json_isodates_on(self, **kwargs):
        return {'date': datetime.datetime.utcnow()}

    @expose('json', render_params={'isodates': False})
    def get_json_isodates_off(self, **kwargs):
        return {'date': datetime.datetime.utcnow()}

    @expose('json', render_params={'allow_lists': True, 'key': 'values'})
    def get_json_list(self, **kwargs):
        return dict(values=[1, 2, 3])

    @expose('json', render_params={'allow_lists': False, 'key': 'values'})
    def json_return_list(self):
        return dict(values=[1,2,3])

    @expose('json')
    @decode_params('json')
    def echo_json(self, **kwargs):
        return kwargs
